@@ -3,10 +3,9 @@ use std::time::Duration;
 use adw::subclass::prelude::*;
 use gettextrs::gettext;
 use gtk::{
-    gdk, gio, glib,
+    CompositeTemplate, gdk, gio, glib,
     glib::{clone, closure, closure_local},
     prelude::*,
-    CompositeTemplate,
 };
 use tracing::{debug, error};
 
@@ -15,12 +14,11 @@ use crate::{
     components::{ActionButton, ActionState, AnimatedImagePaintable},
     toast,
     utils::{
-        expression,
+        BoundObject, BoundObjectWeakRef, CountedRef, SingleItemListModel, expression,
         media::{
-            image::{ImageError, IMAGE_QUEUE},
             FrameDimensions,
+            image::{IMAGE_QUEUE, ImageError},
         },
-        BoundObject, BoundObjectWeakRef, CountedRef,
     },
 };
 
@@ -505,12 +503,11 @@ impl EditableAvatar {
 
     /// Choose a new avatar.
     pub(super) async fn choose_avatar(&self) {
-        let filters = gio::ListStore::new::<gtk::FileFilter>();
-
         let image_filter = gtk::FileFilter::new();
         image_filter.set_name(Some(&gettext("Images")));
         image_filter.add_mime_type("image/*");
-        filters.append(&image_filter);
+
+        let filters = SingleItemListModel::new(Some(&image_filter));
 
         let dialog = gtk::FileDialog::builder()
             .title(gettext("Choose Avatar"))
