@@ -7,7 +7,7 @@ use crate::{
     i18n::gettext_f,
     ngettext_f,
     prelude::*,
-    session::model::{Member, Membership, Room, RoomCategory},
+    session::model::{Member, Membership, Room, RoomCategory, User},
 };
 
 /// Show a dialog to confirm leaving a room.
@@ -23,7 +23,9 @@ pub(crate) async fn confirm_leave_room_dialog(
         // We are rejecting an invite.
         let heading = gettext("Decline Invite?");
         let body = if room.join_rule().we_can_join() {
-            gettext("Do you really want to decline this invite? You can join this room on your own later.")
+            gettext(
+                "Do you really want to decline this invite? You can join this room on your own later.",
+            )
         } else {
             gettext(
                 "Do you really want to decline this invite? You will not be able to join this room without it.",
@@ -132,7 +134,7 @@ impl RoomMemberDestructiveAction {
                     // Translators: Do NOT translate the content between '{' and '}',
                     // this is a variable name.
                     "Are you sure you want to ban {user_id}? They will not be able to join the room again until someone unbans them.",
-                    &[("user_id", member.user_id().as_str())]
+                    &[("user_id", member.user_id().as_str())],
                 );
                 let response = gettext("Ban");
                 (heading, body, Some(response))
@@ -152,16 +154,16 @@ impl RoomMemberDestructiveAction {
                             gettext_f(
                                 // Translators: Do NOT translate the content between '{' and '}',
                                 // this is a variable name.
-                            "Are you sure you want to revoke the invite for {user_id}? They will still be able to join the room on their own.",
-                            &[("user_id", member.user_id().as_str())]
-                        )
+                                "Are you sure you want to revoke the invite for {user_id}? They will still be able to join the room on their own.",
+                                &[("user_id", member.user_id().as_str())],
+                            )
                         } else {
                             gettext_f(
                                 // Translators: Do NOT translate the content between '{' and '}',
                                 // this is a variable name.
-                            "Are you sure you want to revoke the invite for {user_id}? They will not be able to join the room again until someone reinvites them.",
-                            &[("user_id", member.user_id().as_str())]
-                        )
+                                "Are you sure you want to revoke the invite for {user_id}? They will not be able to join the room again until someone reinvites them.",
+                                &[("user_id", member.user_id().as_str())],
+                            )
                         };
                         let response = gettext("Revoke Invite");
                         (heading, body, Some(response))
@@ -191,16 +193,16 @@ impl RoomMemberDestructiveAction {
                             gettext_f(
                                 // Translators: Do NOT translate the content between '{' and '}',
                                 // this is a variable name.
-                            "Are you sure you want to kick {user_id}? They will still be able to join the room again on their own.",
-                            &[("user_id", member.user_id().as_str())]
-                        )
+                                "Are you sure you want to kick {user_id}? They will still be able to join the room again on their own.",
+                                &[("user_id", member.user_id().as_str())],
+                            )
                         } else {
                             gettext_f(
                                 // Translators: Do NOT translate the content between '{' and '}',
                                 // this is a variable name.
-                            "Are you sure you want to kick {user_id}? They will not be able to join the room again until someone invites them.",
-                            &[("user_id", member.user_id().as_str())]
-                        )
+                                "Are you sure you want to kick {user_id}? They will not be able to join the room again until someone invites them.",
+                                &[("user_id", member.user_id().as_str())],
+                            )
                         };
                         let response = gettext("Kick");
                         (heading, body, Some(response))
@@ -217,13 +219,16 @@ impl RoomMemberDestructiveAction {
                         &[("user", &member.display_name())],
                     );
                     let body = ngettext_f(
-                    // Translators: Do NOT translate the content between '{' and '}',
-                    // this is a variable name.
-                    "This removes all the messages received from the homeserver. Are you sure you want to remove 1 message sent by {user_id}? This cannot be undone.",
-                    "This removes all the messages received from the homeserver. Are you sure you want to remove {n} messages sent by {user_id}? This cannot be undone.",
-                    n,
-                    &[("n", &n.to_string()),("user_id", member.user_id().as_str())]
-                );
+                        // Translators: Do NOT translate the content between '{' and '}',
+                        // this is a variable name.
+                        "This removes all the messages received from the homeserver. Are you sure you want to remove 1 message sent by {user_id}? This cannot be undone.",
+                        "This removes all the messages received from the homeserver. Are you sure you want to remove {n} messages sent by {user_id}? This cannot be undone.",
+                        n,
+                        &[
+                            ("n", &n.to_string()),
+                            ("user_id", member.user_id().as_str()),
+                        ],
+                    );
                     let response = gettext("Remove");
                     (heading, body, Some(response))
                 } else {
@@ -234,11 +239,11 @@ impl RoomMemberDestructiveAction {
                         &[("user", &member.display_name())],
                     );
                     let body = gettext_f(
-                    // Translators: Do NOT translate the content between '{' and '}',
-                    // this is a variable name.
-                    "There are no messages received from the homeserver sent by {user_id}. You can try to load more by going further back in the room history.",
-                    &[("user_id", member.user_id().as_str())]
-                );
+                        // Translators: Do NOT translate the content between '{' and '}',
+                        // this is a variable name.
+                        "There are no messages received from the homeserver sent by {user_id}. You can try to load more by going further back in the room history.",
+                        &[("user_id", member.user_id().as_str())],
+                    );
                     (heading, body, None)
                 }
             }
@@ -351,22 +356,36 @@ pub(crate) struct ConfirmRoomMemberDestructiveActionResponse {
     pub remove_events: bool,
 }
 
-/// Show a dialog to confirm muting a room member.
+/// Show a dialog to confirm muting one or several room members.
 pub(crate) async fn confirm_mute_room_member_dialog(
-    member: &Member,
+    members: &[impl IsA<User>],
     parent: &impl IsA<gtk::Widget>,
 ) -> bool {
-    let heading = gettext_f(
+    if members.is_empty() {
+        return false;
+    }
+
+    let first_member = members
+        .first()
+        .expect("there should be at least one member")
+        .upcast_ref();
+    let count = members.len() as u32;
+
+    let heading = ngettext_f(
         // Translators: Do NOT translate the content between '{' and '}',
-        // this is a variable name.
+        // this is a variable name. The count cannot be zero.
         "Mute {user}?",
-        &[("user", &member.display_name())],
+        "Mute Members?",
+        count,
+        &[("user", &first_member.display_name())],
     );
-    let body = gettext_f(
+    let body = ngettext_f(
         // Translators: Do NOT translate the content between '{' and '}',
-        // this is a variable name.
-        "Are you sure you want to mute {user_id}? They will not be able to send new messages.",
-        &[("user_id", member.user_id().as_str())],
+        // this is a variable name. The count cannot be zero.
+        "Are you sure you want to mute {user_id}? They will not be able to send new messages to this room.",
+        "Are you sure you want to mute these members? They will not be able to send new messages to this room.",
+        count,
+        &[("user_id", first_member.user_id().as_str())],
     );
 
     // Ask for confirmation.
@@ -385,23 +404,37 @@ pub(crate) async fn confirm_mute_room_member_dialog(
     confirm_dialog.choose_future(parent).await == "mute"
 }
 
-/// Show a dialog to confirm setting the power level of a room member with the
-/// same value as our own.
+/// Show a dialog to confirm setting the power level of one or several room
+/// members with the same value as our own.
 pub(crate) async fn confirm_set_room_member_power_level_same_as_own_dialog(
-    member: &Member,
+    members: &[impl IsA<User>],
     parent: &impl IsA<gtk::Widget>,
 ) -> bool {
-    let heading = gettext_f(
+    if members.is_empty() {
+        return false;
+    }
+
+    let first_member = members
+        .first()
+        .expect("there should be at least one member")
+        .upcast_ref();
+    let count = members.len() as u32;
+
+    let heading = ngettext_f(
         // Translators: Do NOT translate the content between '{' and '}',
-        // this is a variable name.
+        // this is a variable name. The count cannot be zero.
         "Promote {user}?",
-        &[("user", &member.display_name())],
+        "Promote Members?",
+        count,
+        &[("user", &first_member.display_name())],
     );
-    let body = gettext_f(
+    let body = ngettext_f(
         // Translators: Do NOT translate the content between '{' and '}',
-        // this is a variable name.
+        // this is a variable name. The count cannot be zero.
         "If you promote {user_id} to the same level as yours, you will not be able to demote them in the future.",
-        &[("user_id", member.user_id().as_str())],
+        "If you promote these members to the same level as yours, you will not be able to demote them in the future.",
+        count,
+        &[("user_id", first_member.user_id().as_str())],
     );
 
     // Ask for confirmation.
@@ -418,3 +451,64 @@ pub(crate) async fn confirm_set_room_member_power_level_same_as_own_dialog(
 
     confirm_dialog.choose_future(parent).await == "promote"
 }
+
+/// Show a dialog to confirm the demotion of our own user.
+pub(crate) async fn confirm_own_demotion_dialog(parent: &impl IsA<gtk::Widget>) -> bool {
+    let heading = gettext("Demote Yourself?");
+    let body = gettext(
+        "Are you sure you want to lower your power level? You will need to ask another member with a higher power level to undo this.",
+    );
+
+    // Ask for confirmation.
+    let confirm_dialog = adw::AlertDialog::builder()
+        .default_response("cancel")
+        .heading(heading)
+        .body(body)
+        .build();
+    confirm_dialog.add_responses(&[
+        ("cancel", &gettext("Cancel")),
+        ("demote", &gettext("Demote")),
+    ]);
+    confirm_dialog.set_response_appearance("demote", adw::ResponseAppearance::Destructive);
+
+    confirm_dialog.choose_future(parent).await == "demote"
+}
+
+/// Show a dialog for the user to choose what to do about unsaved changes.
+pub(crate) async fn unsaved_changes_dialog(
+    parent: &impl IsA<gtk::Widget>,
+) -> UnsavedChangesResponse {
+    let title = gettext("Save Changes?");
+    let description =
+        gettext("This page contains unsaved changes. Changes which are not saved will be lost.");
+    let dialog = adw::AlertDialog::builder()
+        .title(title)
+        .body(description)
+        .default_response("cancel")
+        .build();
+
+    dialog.add_responses(&[
+        ("cancel", &gettext("Cancel")),
+        ("discard", &gettext("Discard")),
+        ("save", &gettext("Save")),
+    ]);
+    dialog.set_response_appearance("discard", adw::ResponseAppearance::Destructive);
+    dialog.set_response_appearance("save", adw::ResponseAppearance::Suggested);
+
+    match dialog.choose_future(parent).await.as_str() {
+        "discard" => UnsavedChangesResponse::Discard,
+        "save" => UnsavedChangesResponse::Save,
+        _ => UnsavedChangesResponse::Cancel,
+    }
+}
+
+/// A response to the dialog to choose what to do about unsaved changes.
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub(crate) enum UnsavedChangesResponse {
+    /// Save the changes.
+    Save,
+    /// Discard the changes.
+    Discard,
+    /// Cancel the current action.
+    Cancel,
+}
