@@ -1,5 +1,5 @@
 use adw::{prelude::*, subclass::prelude::*};
-use gtk::{glib, glib::clone, CompositeTemplate};
+use gtk::{CompositeTemplate, glib, glib::clone};
 
 mod at_room;
 mod search_entry;
@@ -12,18 +12,21 @@ pub use self::{
     source::{PillSource, PillSourceExt, PillSourceImpl},
     source_row::PillSourceRow,
 };
-use super::{Avatar, RoomPreviewDialog, UserProfileDialog};
+use super::{Avatar, AvatarImageSafetySetting, RoomPreviewDialog, UserProfileDialog};
 use crate::{
     prelude::*,
     session::{
         model::{Member, RemoteRoom, Room},
         view::SessionView,
     },
-    utils::{key_bindings, BoundObject},
+    utils::{BoundObject, key_bindings},
 };
 
 mod imp {
-    use std::cell::{Cell, RefCell};
+    use std::{
+        cell::{Cell, RefCell},
+        marker::PhantomData,
+    };
 
     use glib::subclass::InitializingObject;
 
@@ -45,6 +48,15 @@ mod imp {
         /// Whether the pill can be activated.
         #[property(get, set = Self::set_activatable, explicit_notify)]
         activatable: Cell<bool>,
+        /// The safety setting to watch to decide whether the image of the
+        /// avatar should be displayed.
+        #[property(get = Self::watched_safety_setting, set = Self::set_watched_safety_setting, builder(AvatarImageSafetySetting::default()))]
+        watched_safety_setting: PhantomData<AvatarImageSafetySetting>,
+        /// The room to watch to apply the current safety settings.
+        ///
+        /// This is required if `watched_safety_setting` is not `None`.
+        #[property(get = Self::watched_room, set = Self::set_watched_room, nullable)]
+        watched_room: PhantomData<Option<Room>>,
         gesture_click: RefCell<Option<gtk::GestureClick>>,
     }
 
@@ -165,6 +177,28 @@ mod imp {
             }
         }
 
+        /// The safety setting to watch to decide whether the image of the
+        /// avatar should be displayed.
+        fn watched_safety_setting(&self) -> AvatarImageSafetySetting {
+            self.avatar.watched_safety_setting()
+        }
+
+        /// Set the safety setting to watch to decide whether the image of the
+        /// avatar should be displayed.
+        fn set_watched_safety_setting(&self, setting: AvatarImageSafetySetting) {
+            self.avatar.set_watched_safety_setting(setting);
+        }
+
+        /// The room to watch to apply the current safety settings.
+        fn watched_room(&self) -> Option<Room> {
+            self.avatar.watched_room()
+        }
+
+        /// Set the room to watch to apply the current safety settings.
+        fn set_watched_room(&self, room: Option<Room>) {
+            self.avatar.set_watched_room(room);
+        }
+
         /// Set the display name of this pill.
         fn set_display_name(&self, label: &str) {
             // We ellipsize the string manually because GtkTextView uses the minimum width.
@@ -222,8 +256,30 @@ glib::wrapper! {
 }
 
 impl Pill {
-    /// Create a pill with the given source.
-    pub fn new(source: &impl IsA<PillSource>) -> Self {
-        glib::Object::builder().property("source", source).build()
+    /// Create a pill with the given source and watching the given safety
+    /// setting.
+    pub fn new(
+        source: &impl IsA<PillSource>,
+        watched_safety_setting: AvatarImageSafetySetting,
+        watched_room: Option<Room>,
+    ) -> Self {
+        let source = source.upcast_ref();
+
+        let (watched_safety_setting, watched_room) = if let Some(room) = source
+            .downcast_ref::<Room>()
+            .cloned()
+            .or_else(|| source.downcast_ref::<AtRoom>().map(AtRoom::room))
+        {
+            // We must always watch the invite avatars setting for local rooms.
+            (AvatarImageSafetySetting::InviteAvatars, Some(room))
+        } else {
+            (watched_safety_setting, watched_room)
+        };
+
+        glib::Object::builder()
+            .property("source", source)
+            .property("watched-safety-setting", watched_safety_setting)
+            .property("watched-room", watched_room)
+            .build()
     }
 }
