@@ -1,11 +1,10 @@
 use adw::{prelude::*, subclass::prelude::*};
 use gtk::{
-    glib,
+    CompositeTemplate, glib,
     glib::{clone, closure_local},
-    CompositeTemplate,
 };
 
-use crate::components::{Pill, PillSource};
+use crate::components::{AvatarImageSafetySetting, Pill, PillSource};
 
 mod imp {
     use std::{cell::RefCell, collections::HashMap, marker::PhantomData, sync::LazyLock};
@@ -53,9 +52,11 @@ mod imp {
     impl ObjectImpl for PillSearchEntry {
         fn signals() -> &'static [Signal] {
             static SIGNALS: LazyLock<Vec<Signal>> = LazyLock::new(|| {
-                vec![Signal::builder("pill-removed")
-                    .param_types([PillSource::static_type()])
-                    .build()]
+                vec![
+                    Signal::builder("pill-removed")
+                        .param_types([PillSource::static_type()])
+                        .build(),
+                ]
             });
             SIGNALS.as_ref()
         }
@@ -175,7 +176,9 @@ mod imp {
                 return;
             }
 
-            let pill = Pill::new(source);
+            // We do not need to watch the safety setting as this entry should only be used
+            // with search results.
+            let pill = Pill::new(source, AvatarImageSafetySetting::None, None);
             pill.set_margin_start(3);
             pill.set_margin_end(3);
 
