@@ -3,12 +3,13 @@ pub(crate) use crate::{
         CameraExt, ContextMenuBinExt, ContextMenuBinImpl, PillSourceExt, PillSourceImpl,
         ToastableDialogExt, ToastableDialogImpl,
     },
+    secret::SecretExt,
     session::model::{TimelineItemExt, UserExt},
     session_list::SessionInfoExt,
     user_facing_error::UserFacingError,
     utils::{
+        ChildPropertyExt, IsABin, LocationExt,
         matrix::ext_traits::*,
-        string::{StrExt, StrMutExt},
-        LocationExt,
+        string::{OptionStringExt, StrExt, StrMutExt},
     },
 };
