@@ -3,13 +3,11 @@ use std::{cell::RefCell, time::Duration};
 use gtk::{glib, glib::clone, prelude::*, subclass::prelude::*};
 use matrix_sdk::reqwest::StatusCode;
 use ruma::{
-    api::client::{
-        room::get_summary,
-        space::{get_hierarchy, SpaceHierarchyRoomsChunk},
-    },
+    OwnedRoomAliasId, OwnedRoomId,
+    api::client::{room::get_summary, space::get_hierarchy},
     assign,
-    directory::PublicRoomsChunk,
-    uint, OwnedMxcUri, OwnedRoomAliasId, OwnedRoomId,
+    room::{JoinRuleSummary, RoomSummary},
+    uint,
 };
 use tracing::{debug, warn};
 
@@ -18,7 +16,7 @@ use crate::{
     prelude::*,
     session::model::{RoomListRoomInfo, Session},
     spawn, spawn_tokio,
-    utils::{matrix::MatrixRoomIdUri, string::linkify, AbortableHandle, LoadingState},
+    utils::{AbortableHandle, LoadingState, matrix::MatrixRoomIdUri, string::linkify},
 };
 
 /// The time after which the data of a room is assumed to be stale.
@@ -64,6 +62,9 @@ mod imp {
         /// The number of joined members in the room.
         #[property(get)]
         joined_members_count: Cell<u32>,
+        /// Whether we can knock on the room.
+        #[property(get)]
+        can_knock: Cell<bool>,
         /// The information about this room in the room list.
         #[property(get)]
         room_list_info: RoomListRoomInfo,
@@ -228,7 +229,7 @@ mod imp {
             obj.notify_topic_linkified();
         }
 
-        /// Set the loading state.
+        /// Set the number of joined members in the room.
         fn set_joined_members_count(&self, count: u32) {
             if self.joined_members_count.get() == count {
                 return;
@@ -238,6 +239,21 @@ mod imp {
             self.obj().notify_joined_members_count();
         }
 
+        /// Set the join rule of the room.
+        fn set_join_rule(&self, join_rule: &JoinRuleSummary) {
+            let can_knock = matches!(
+                join_rule,
+                JoinRuleSummary::Knock | JoinRuleSummary::KnockRestricted(_)
+            );
+
+            if self.can_knock.get() == can_knock {
+                return;
+            }
+
+            self.can_knock.set(can_knock);
+            self.obj().notify_can_knock();
+        }
+
         /// Set the loading state.
         pub(super) fn set_loading_state(&self, loading_state: LoadingState) {
             if self.loading_state.get() == loading_state {
@@ -255,12 +271,13 @@ mod imp {
         }
 
         /// Set the room data.
-        pub(super) fn set_data(&self, data: RemoteRoomData) {
+        pub(super) fn set_data(&self, data: RoomSummary) {
             self.set_room_id(data.room_id);
             self.set_canonical_alias(data.canonical_alias);
-            self.set_name(data.name);
-            self.set_topic(data.topic);
-            self.set_joined_members_count(data.joined_members_count);
+            self.set_name(data.name.into_clean_string());
+            self.set_topic(data.topic.into_clean_string());
+            self.set_joined_members_count(data.num_joined_members.try_into().unwrap_or(u32::MAX));
+            self.set_join_rule(&data.join_rule);
 
             if let Some(image) = self.obj().avatar_data().image() {
                 image.set_uri_and_info(data.avatar_url, None);
@@ -308,7 +325,7 @@ mod imp {
             let uri = self.uri();
             let client = session.client();
 
-            let request = get_summary::msc3266::Request::new(uri.id.clone(), uri.via.clone());
+            let request = get_summary::v1::Request::new(uri.id.clone(), uri.via.clone());
             let handle = spawn_tokio!(async move { client.send(request).await });
 
             let Some(result) = self.request_abort_handle.await_task(handle).await else {
@@ -318,7 +335,7 @@ mod imp {
 
             match result {
                 Ok(response) => {
-                    self.set_data(response.into());
+                    self.set_data(response.summary);
                     true
                 }
                 Err(error) => {
@@ -388,9 +405,9 @@ mod imp {
                         .rooms
                         .into_iter()
                         .next()
-                        .filter(|c| c.room_id == room_id)
+                        .filter(|c| c.summary.room_id == room_id)
                     {
-                        self.set_data(chunk.into());
+                        self.set_data(chunk.summary);
                     } else {
                         debug!("Space hierarchy endpoint did not return requested room");
                         self.set_loading_state(LoadingState::Error);
@@ -437,7 +454,7 @@ impl RemoteRoom {
     pub(crate) fn with_data(
         session: &Session,
         uri: MatrixRoomIdUri,
-        data: impl Into<RemoteRoomData>,
+        data: impl Into<RoomSummary>,
     ) -> Self {
         let obj = Self::without_data(session, uri);
         obj.imp().set_data(data.into());
@@ -482,53 +499,3 @@ impl RemoteRoom {
         ));
     }
 }
-
-/// The remote room data.
-#[derive(Debug)]
-pub(crate) struct RemoteRoomData {
-    room_id: OwnedRoomId,
-    canonical_alias: Option<OwnedRoomAliasId>,
-    name: Option<String>,
-    topic: Option<String>,
-    avatar_url: Option<OwnedMxcUri>,
-    joined_members_count: u32,
-}
-
-impl From<get_summary::msc3266::Response> for RemoteRoomData {
-    fn from(value: get_summary::msc3266::Response) -> Self {
-        Self {
-            room_id: value.room_id,
-            canonical_alias: value.canonical_alias,
-            name: value.name,
-            topic: value.topic,
-            avatar_url: value.avatar_url,
-            joined_members_count: value.num_joined_members.try_into().unwrap_or(u32::MAX),
-        }
-    }
-}
-
-impl From<SpaceHierarchyRoomsChunk> for RemoteRoomData {
-    fn from(value: SpaceHierarchyRoomsChunk) -> Self {
-        Self {
-            room_id: value.room_id,
-            canonical_alias: value.canonical_alias,
-            name: value.name,
-            topic: value.topic,
-            avatar_url: value.avatar_url,
-            joined_members_count: value.num_joined_members.try_into().unwrap_or(u32::MAX),
-        }
-    }
-}
-
-impl From<PublicRoomsChunk> for RemoteRoomData {
-    fn from(value: PublicRoomsChunk) -> Self {
-        Self {
-            room_id: value.room_id,
-            canonical_alias: value.canonical_alias,
-            name: value.name,
-            topic: value.topic,
-            avatar_url: value.avatar_url,
-            joined_members_count: value.num_joined_members.try_into().unwrap_or(u32::MAX),
-        }
-    }
-}
