@@ -3,16 +3,16 @@ use std::sync::Arc;
 use gtk::{gio, glib, glib::closure_local, prelude::*, subclass::prelude::*};
 use indexmap::IndexMap;
 use matrix_sdk_ui::timeline::{
-    AnyOtherFullStateEventContent, Error as TimelineError, EventSendState, EventTimelineItem,
-    MembershipChange, Message, MsgLikeKind, RepliedToEvent, TimelineDetails, TimelineEventItemId,
-    TimelineItemContent,
+    AnyOtherFullStateEventContent, EmbeddedEvent, Error as TimelineError, EventSendState,
+    EventTimelineItem, MembershipChange, Message, MsgLikeKind, TimelineDetails,
+    TimelineEventItemId, TimelineItemContent,
 };
 use ruma::{
-    events::{receipt::Receipt, AnySyncTimelineEvent, TimelineEventType},
-    serde::Raw,
     MatrixToUri, MilliSecondsSinceUnixEpoch, OwnedEventId, OwnedTransactionId, OwnedUserId, UserId,
+    events::{AnySyncTimelineEvent, TimelineEventType, receipt::Receipt},
+    serde::Raw,
 };
-use serde::{de::IgnoredAny, Deserialize};
+use serde::{Deserialize, de::IgnoredAny};
 use tracing::{debug, error};
 
 mod reaction_group;
@@ -27,7 +27,7 @@ use crate::{
     prelude::*,
     session::model::Member,
     spawn_tokio,
-    utils::matrix::{raw_eq, timestamp_to_date, MediaMessage, VisualMediaMessage},
+    utils::matrix::{MediaMessage, raw_eq, timestamp_to_date},
 };
 
 /// The possible states of a message.
@@ -187,7 +187,7 @@ mod imp {
             let item = Arc::new(item);
             let prev_item = self.item.replace(Some(item.clone()));
 
-            self.reactions.update(&item.content().reactions());
+            self.reactions.update(item.content().reactions());
             self.update_read_receipts(item.read_receipts());
 
             let prev_source = prev_item.as_ref().and_then(|i| i.original_json());
@@ -299,7 +299,10 @@ mod imp {
         /// The pretty-formatted JSON source, if it has been echoed back by the
         /// server.
         fn source(&self) -> Option<String> {
-            self.item().original_json().map(raw_to_pretty_string)
+            self.item()
+                .original_json()
+                .map(raw_to_pretty_string)
+                .into_clean_string()
         }
 
         /// Whether we have the JSON source.
@@ -313,7 +316,7 @@ mod imp {
 
             if let Some(send_state) = item.send_state() {
                 match send_state {
-                    EventSendState::NotSentYet => return MessageState::Sending,
+                    EventSendState::NotSentYet { .. } => return MessageState::Sending,
                     EventSendState::SendingFailed {
                         error,
                         is_recoverable,
@@ -373,6 +376,7 @@ mod imp {
             self.latest_edit_raw()
                 .as_ref()
                 .map(raw_to_pretty_string)
+                .into_clean_string()
                 .unwrap_or_default()
         }
 
@@ -658,12 +662,11 @@ impl Event {
                 true
             }
             TimelineItemContent::OtherState(other_state) => {
-                // `m.room.create` and `m.room.tombstone` should only occur once per room and
-                // they have special rendering so we do not group them.
+                // `m.room.create` should only occur once per room and it has special rendering
+                // so we do not group it.
                 !matches!(
                     other_state.content(),
                     AnyOtherFullStateEventContent::RoomCreate(_)
-                        | AnyOtherFullStateEventContent::RoomTombstone(_)
                 )
             }
             _ => false,
@@ -703,19 +706,6 @@ impl Event {
         }
     }
 
-    /// The visual media message of this event, if any.
-    pub(crate) fn visual_media_message(&self) -> Option<VisualMediaMessage> {
-        match self.item().content() {
-            TimelineItemContent::MsgLike(msg_like) => match &msg_like.kind {
-                MsgLikeKind::Message(message) => {
-                    VisualMediaMessage::from_message(message.msgtype())
-                }
-                _ => None,
-            },
-            _ => None,
-        }
-    }
-
     /// Whether this event might contain an `@room` mention.
     ///
     /// This means that either it does not have intentional mentions, or it has
@@ -742,7 +732,7 @@ impl Event {
     /// Get the details of the event this event replies to, if any.
     ///
     /// Returns `None(_)` if this event is not a reply.
-    pub(crate) fn reply_to_event_content(&self) -> Option<TimelineDetails<Box<RepliedToEvent>>> {
+    pub(crate) fn reply_to_event_content(&self) -> Option<TimelineDetails<Box<EmbeddedEvent>>> {
         match self.item().content() {
             TimelineItemContent::MsgLike(msg_like) => {
                 msg_like.in_reply_to.as_ref().map(|d| d.event.clone())
