@@ -10,16 +10,17 @@ use gtk::{
 use matrix_sdk_ui::{
     eyeball_im::VectorDiff,
     timeline::{
-        default_event_filter, RoomExt, Timeline as SdkTimeline, TimelineEventItemId,
-        TimelineItem as SdkTimelineItem,
+        RoomExt, Timeline as SdkTimeline, TimelineEventItemId, TimelineItem as SdkTimelineItem,
+        default_event_filter,
     },
 };
 use ruma::{
+    OwnedEventId, UserId,
     events::{
-        room::message::MessageType, AnySyncMessageLikeEvent, AnySyncStateEvent,
-        AnySyncTimelineEvent, SyncMessageLikeEvent, SyncStateEvent,
+        AnySyncMessageLikeEvent, AnySyncStateEvent, AnySyncTimelineEvent, SyncMessageLikeEvent,
+        SyncStateEvent, room::message::MessageType,
     },
-    OwnedEventId, RoomVersionId, UserId,
+    room_version_rules::RoomVersionRules,
 };
 use tokio::task::AbortHandle;
 use tracing::error;
@@ -267,7 +268,7 @@ mod imp {
         /// Items added at the start of the timeline.
         fn start_items(&self) -> &SingleItemListModel {
             self.start_items.get_or_init(|| {
-                let model = SingleItemListModel::new(&VirtualItem::spinner(&self.obj()));
+                let model = SingleItemListModel::new(Some(&VirtualItem::spinner(&self.obj())));
                 model.set_is_hidden(true);
                 model
             })
@@ -285,7 +286,7 @@ mod imp {
         /// Items added at the end of the timeline.
         fn end_items(&self) -> &SingleItemListModel {
             self.end_items.get_or_init(|| {
-                let model = SingleItemListModel::new(&VirtualItem::typing(&self.obj()));
+                let model = SingleItemListModel::new(Some(&VirtualItem::typing(&self.obj())));
                 model.set_is_hidden(true);
                 model
             })
@@ -1094,9 +1095,9 @@ impl Timeline {
 }
 
 /// Whether the given event should be shown in the timeline.
-fn show_in_timeline(any: &AnySyncTimelineEvent, room_version: &RoomVersionId) -> bool {
+fn show_in_timeline(any: &AnySyncTimelineEvent, rules: &RoomVersionRules) -> bool {
     // Make sure we do not show events that cannot be shown.
-    if !default_event_filter(any, room_version) {
+    if !default_event_filter(any, rules) {
         return false;
     }
 
@@ -1143,7 +1144,6 @@ fn show_in_timeline(any: &AnySyncTimelineEvent, room_version: &RoomVersionId) ->
                 | AnySyncStateEvent::RoomCreate(_)
                 | AnySyncStateEvent::RoomEncryption(_)
                 | AnySyncStateEvent::RoomThirdPartyInvite(_)
-                | AnySyncStateEvent::RoomTombstone(_)
         ),
     }
 }
