@@ -1,6 +1,8 @@
 use std::{
     cell::Cell,
     collections::{HashMap, HashSet},
+    rc::Rc,
+    time::Duration,
 };
 
 use gtk::{
@@ -49,6 +51,7 @@ mod imp {
         /// The rooms metainfo that allow to restore this `RoomList` from its
         /// previous state.
         metainfo: RoomListMetainfo,
+        pub(super) get_wait_source: RefCell<Option<glib::SourceId>>,
     }
 
     #[glib::object_subclass]
@@ -70,6 +73,12 @@ mod imp {
             self.parent_constructed();
             self.metainfo.set_room_list(&self.obj());
         }
+
+        fn dispose(&self) {
+            if let Some(source) = self.get_wait_source.take() {
+                source.remove();
+            }
+        }
     }
 
     impl ListModelImpl for RoomList {
@@ -210,7 +219,7 @@ mod imp {
 
             let mut new_rooms = HashMap::new();
 
-            for (room_id, left_room) in rooms.leave {
+            for (room_id, left_room) in rooms.left {
                 let room = if let Some(room) = self.get(&room_id) {
                     room
                 } else if let Some(matrix_room) = client.get_room(&room_id) {
@@ -227,7 +236,7 @@ mod imp {
                 room.handle_ambiguity_changes(left_room.ambiguity_changes.values());
             }
 
-            for (room_id, joined_room) in rooms.join {
+            for (room_id, joined_room) in rooms.joined {
                 let room = if let Some(room) = self.get(&room_id) {
                     room
                 } else if let Some(matrix_room) = client.get_room(&room_id) {
@@ -245,7 +254,7 @@ mod imp {
                 room.handle_ambiguity_changes(joined_room.ambiguity_changes.values());
             }
 
-            for (room_id, _invited_room) in rooms.invite {
+            for (room_id, _invited_room) in rooms.invited {
                 let room = if let Some(room) = self.get(&room_id) {
                     room
                 } else if let Some(matrix_room) = client.get_room(&room_id) {
@@ -262,6 +271,23 @@ mod imp {
                 self.metainfo.watch_room(&room);
             }
 
+            for (room_id, _knocked_room) in rooms.knocked {
+                let room = if let Some(room) = self.get(&room_id) {
+                    room
+                } else if let Some(matrix_room) = client.get_room(&room_id) {
+                    new_rooms
+                        .entry(room_id.clone())
+                        .or_insert_with(|| Room::new(&session, matrix_room, None))
+                        .clone()
+                } else {
+                    warn!("Could not find knocked room {room_id}");
+                    continue;
+                };
+
+                self.remove_joining_room((*room_id).into());
+                self.metainfo.watch_room(&room);
+            }
+
             if !new_rooms.is_empty() {
                 let added = new_rooms.len();
                 self.list.borrow_mut().extend(new_rooms);
@@ -309,6 +335,38 @@ mod imp {
                 }
             }
         }
+
+        /// Request an invite.
+        pub(super) async fn knock(
+            &self,
+            identifier: OwnedRoomOrAliasId,
+            via: Vec<OwnedServerName>,
+        ) -> Result<OwnedRoomId, String> {
+            let Some(session) = self.session.upgrade() else {
+                return Err("Could not upgrade Session".to_owned());
+            };
+            let client = session.client();
+
+            let identifier_clone = identifier.clone();
+            let handle =
+                spawn_tokio!(async move { client.knock(identifier_clone, None, via).await });
+
+            match handle.await.expect("task was not aborted") {
+                Ok(matrix_room) => Ok(matrix_room.room_id().to_owned()),
+                Err(error) => {
+                    error!("Invite request for room {identifier} failed: {error}");
+
+                    let error = gettext_f(
+                        // Translators: Do NOT translate the content between '{' and '}', this is a
+                        // variable name.
+                        "Could not request an invite to room {room_name}",
+                        &[("room_name", identifier.as_str())],
+                    );
+
+                    Err(error)
+                }
+            }
+        }
     }
 }
 
@@ -405,28 +463,51 @@ impl RoomList {
     }
 
     /// Wait till the room with the given ID becomes available.
-    pub(crate) async fn get_wait(&self, room_id: &RoomId) -> Option<Room> {
+    pub(crate) async fn get_wait(
+        &self,
+        room_id: &RoomId,
+        timeout: Option<Duration>,
+    ) -> Option<Room> {
         if let Some(room) = self.get(room_id) {
             return Some(room);
         }
 
+        let imp = self.imp();
         let (sender, receiver) = futures_channel::oneshot::channel();
 
         let room_id = room_id.to_owned();
-        let sender = Cell::new(Some(sender));
-        // FIXME: add a timeout
-        let handler_id = self.connect_items_changed(move |obj, _, _, _| {
-            if let Some(room) = obj.get(&room_id) {
-                if let Some(sender) = sender.take() {
-                    let _ = sender.send(Some(room));
+        let sender_cell = Rc::new(Cell::new(Some(sender)));
+
+        let handler_id = self.connect_items_changed(clone!(
+            #[strong]
+            sender_cell,
+            move |obj, _, _, _| {
+                if let Some(room) = obj.get(&room_id) {
+                    if let Some(sender) = sender_cell.take() {
+                        let _ = sender.send(Some(room));
+                    }
                 }
             }
-        });
+        ));
+
+        if let Some(timeout) = timeout {
+            let get_wait_source = glib::timeout_add_local_once(timeout, move || {
+                if let Some(sender) = sender_cell.take() {
+                    let _ = sender.send(None);
+                }
+            });
+            imp.get_wait_source.replace(Some(get_wait_source));
+        }
 
         let room = receiver.await.ok().flatten();
 
         self.disconnect(handler_id);
 
+        // Remove the source if we got a room.
+        if let Some(source) = imp.get_wait_source.take().filter(|_| room.is_some()) {
+            source.remove();
+        }
+
         room
     }
 
@@ -468,6 +549,15 @@ impl RoomList {
         self.imp().join_by_id_or_alias(identifier, via).await
     }
 
+    /// Request an invite to the room with the given identifier.
+    pub(crate) async fn knock(
+        &self,
+        identifier: OwnedRoomOrAliasId,
+        via: Vec<OwnedServerName>,
+    ) -> Result<OwnedRoomId, String> {
+        self.imp().knock(identifier, via).await
+    }
+
     /// Connect to the signal emitted when the list of rooms we are currently
     /// joining changed.
     pub fn connect_joining_rooms_changed<F: Fn(&Self) + 'static>(
