@@ -1,10 +1,10 @@
 use std::time::Duration;
 
-use futures_util::{lock::Mutex, StreamExt};
+use futures_util::{StreamExt, lock::Mutex};
 use gettextrs::gettext;
 use gtk::{gio, glib, glib::clone, prelude::*, subclass::prelude::*};
 use matrix_sdk::{
-    config::SyncSettings, media::MediaRetentionPolicy, sync::SyncResponse, Client, SessionChange,
+    Client, SessionChange, config::SyncSettings, media::MediaRetentionPolicy, sync::SyncResponse,
 };
 use ruma::{
     api::client::{
@@ -18,20 +18,20 @@ use tokio_stream::wrappers::BroadcastStream;
 use tracing::{debug, error, info};
 
 use super::{
-    IgnoredUsers, Notifications, RemoteCache, RoomList, SessionSecurity, SessionSettings,
-    SidebarItemList, SidebarListModel, User, UserSessionsList, VerificationList,
+    GlobalAccountData, IgnoredUsers, Notifications, RemoteCache, RoomList, SessionSecurity,
+    SessionSettings, SidebarItemList, SidebarListModel, User, UserSessionsList, VerificationList,
 };
 use crate::{
+    Application,
     components::AvatarData,
     prelude::*,
     secret::StoredSession,
     session_list::{SessionInfo, SessionInfoImpl},
     spawn, spawn_tokio,
     utils::{
-        matrix::{self, ClientSetupError},
         TokioDrop,
+        matrix::{self, ClientSetupError},
     },
-    Application,
 };
 
 /// The database key for persisting the session's profile.
@@ -91,6 +91,9 @@ mod imp {
         /// The current settings for this session.
         #[property(get, construct_only)]
         settings: OnceCell<SessionSettings>,
+        /// The settings in the global account data for this session.
+        #[property(get = Self::global_account_data_owned)]
+        global_account_data: OnceCell<GlobalAccountData>,
         /// The notifications API for this session.
         #[property(get)]
         notifications: Notifications,
@@ -335,8 +338,20 @@ mod imp {
             self.obj().notify_is_offline();
         }
 
+        /// The settings stored in the global account data for this session.
+        fn global_account_data(&self) -> &GlobalAccountData {
+            self.global_account_data
+                .get_or_init(|| GlobalAccountData::new(&self.obj()))
+        }
+
+        /// The owned settings stored in the global account data for this
+        /// session.
+        fn global_account_data_owned(&self) -> GlobalAccountData {
+            self.global_account_data().clone()
+        }
+
         /// The cache for remote data.
-        pub(crate) fn remote_cache(&self) -> &RemoteCache {
+        pub(super) fn remote_cache(&self) -> &RemoteCache {
             self.remote_cache
                 .get_or_init(|| RemoteCache::new(self.obj().clone()))
         }
@@ -356,6 +371,8 @@ mod imp {
                     }
                 )
             );
+
+            self.global_account_data();
             self.watch_session_changes();
             self.update_homeserver_reachable().await;
 
@@ -430,6 +447,12 @@ mod imp {
             let obj_weak = glib::SendWeakRef::from(self.obj().downgrade());
 
             let handle = spawn_tokio!(async move {
+                // Make sure that the event cache is subscribed to sync responses to benefit
+                // from it.
+                if let Err(error) = client.event_cache().subscribe() {
+                    error!("Could not subscribe event cache to sync responses: {error}");
+                }
+
                 // TODO: only create the filter once and reuse it in the future
                 let filter = assign!(FilterDefinition::default(), {
                     room: assign!(RoomFilter::with_lazy_loading(), {
@@ -439,6 +462,7 @@ mod imp {
 
                 let sync_settings = SyncSettings::new()
                     .timeout(Duration::from_secs(30))
+                    .ignore_timeout_on_first_sync(true)
                     .filter(filter.into());
 
                 let mut sync_stream = Box::pin(client.sync_stream(sync_settings).await);
@@ -534,7 +558,7 @@ mod imp {
                     Err(error) => {
                         error!(
                             session = self.obj().session_id(),
-                            "Failed to deserialize session profile: {error}"
+                            "Could not deserialize session profile: {error}"
                         );
                         return;
                     }
@@ -605,7 +629,7 @@ mod imp {
                 Err(error) => {
                     error!(
                         session = self.obj().session_id(),
-                        "Failed to serialize session profile: {error}"
+                        "Could not serialize session profile: {error}"
                     );
                     return;
                 }
