@@ -185,7 +185,7 @@ mod imp {
                     let room = self
                         .session()
                         .room_list()
-                        .get_wait(matrix_room.room_id())
+                        .get_wait(matrix_room.room_id(), None)
                         .await
                         .expect("The newly created room was not found");
                     Ok(room)
@@ -371,7 +371,7 @@ pub trait UserExt: IsA<User> {
             Ok(response) => {
                 let user = self.upcast_ref::<User>();
 
-                user.set_name(response.displayname);
+                user.set_name(response.displayname.into_clean_string());
                 user.set_avatar_url(response.avatar_url);
                 Ok(())
             }
