@@ -1,25 +1,21 @@
 use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::gettext;
-use gtk::{
-    gio,
-    glib::{self, clone},
-    CompositeTemplate,
-};
+use gtk::{CompositeTemplate, gio, glib, glib::clone};
 use matrix_sdk::RoomState;
-use ruma::{assign, events::room::avatar::ImageInfo, OwnedMxcUri};
+use ruma::{OwnedMxcUri, assign, events::room::avatar::ImageInfo};
 use tracing::error;
 
 use crate::{
     components::{
         ActionButton, ActionState, AvatarData, AvatarImage, EditableAvatar, LoadingButton,
+        UnsavedChangesResponse, unsaved_changes_dialog,
     },
     prelude::*,
     session::model::Room,
     spawn_tokio, toast,
     utils::{
-        media::{image::ImageInfoLoader, FileInfo},
-        template_callbacks::TemplateCallbacks,
-        BoundObjectWeakRef, OngoingAsyncAction,
+        BoundObjectWeakRef, OngoingAsyncAction, TemplateCallbacks,
+        media::{FileInfo, image::ImageInfoLoader},
     },
 };
 
@@ -142,7 +138,7 @@ mod imp {
             } else {
                 // No action is ongoing, we don't need to do anything.
                 return;
-            };
+            }
 
             // Reset the state.
             self.changing_avatar.take();
@@ -325,10 +321,9 @@ mod imp {
             } else {
                 // No action is ongoing, we don't need to do anything.
                 return;
-            };
+            }
 
-            let obj = self.obj();
-            toast!(obj, gettext("Room name saved successfully"));
+            toast!(self.obj(), gettext("Room name saved successfully"));
 
             // Reset state.
             self.changing_name.take();
@@ -399,8 +394,7 @@ mod imp {
                 if weak_action.is_ongoing() {
                     self.changing_name.take();
                     error!("Could not change room name: {error}");
-                    let obj = self.obj();
-                    toast!(obj, gettext("Could not change room name"));
+                    toast!(self.obj(), gettext("Could not change room name"));
                     self.name_entry_row.set_sensitive(true);
                     self.name_button.set_state(ActionState::Retry);
                 }
@@ -436,10 +430,9 @@ mod imp {
             } else {
                 // No action is ongoing, we don't need to do anything.
                 return;
-            };
+            }
 
-            let obj = self.obj();
-            toast!(obj, gettext("Room description saved successfully"));
+            toast!(self.obj(), gettext("Room description saved successfully"));
 
             // Reset state.
             self.changing_topic.take();
@@ -515,8 +508,7 @@ mod imp {
                 if weak_action.is_ongoing() {
                     self.changing_topic.take();
                     error!("Could not change room description: {error}");
-                    let obj = self.obj();
-                    toast!(obj, gettext("Could not change room description"));
+                    toast!(self.obj(), gettext("Could not change room description"));
                     self.topic_text_view.set_sensitive(true);
                     self.save_topic_button.set_is_loading(false);
                 }
@@ -531,34 +523,23 @@ mod imp {
             let obj = self.obj();
             let mut reset_after = false;
 
-            let has_unsaved_changes = (self.was_name_edited()
-                && self.changing_name.borrow().is_none())
-                || (self.was_topic_edited() && self.changing_topic.borrow().is_none());
+            let name_was_edited = self.was_name_edited() && self.changing_name.borrow().is_none();
+            let topic_was_edited =
+                self.was_topic_edited() && self.changing_topic.borrow().is_none();
 
-            if has_unsaved_changes {
-                let title = gettext("Discard Unsaved Changes?");
-                let description = gettext(
-                    "This page contains unsaved changes. Changes which are not saved will be lost.",
-                );
-                let dialog = adw::AlertDialog::builder()
-                    .title(title)
-                    .body(description)
-                    .default_response("cancel")
-                    .build();
+            if name_was_edited || topic_was_edited {
+                match unsaved_changes_dialog(&*obj).await {
+                    UnsavedChangesResponse::Save => {
+                        if name_was_edited {
+                            self.change_name().await;
+                        }
 
-                dialog.add_responses(&[
-                    ("cancel", &gettext("Cancel")),
-                    ("discard", &gettext("Discard")),
-                ]);
-                dialog.set_response_appearance("discard", adw::ResponseAppearance::Destructive);
-
-                match dialog.choose_future(&*obj).await.as_str() {
-                    "discard" => {
-                        reset_after = true;
-                    }
-                    _ => {
-                        return;
+                        if topic_was_edited {
+                            self.change_topic().await;
+                        }
                     }
+                    UnsavedChangesResponse::Discard => reset_after = true,
+                    UnsavedChangesResponse::Cancel => return,
                 }
             }
 
