@@ -1,41 +1,42 @@
 use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::{gettext, ngettext};
 use gtk::{
-    gio,
+    CompositeTemplate, gio,
     glib::{self, clone},
-    pango, CompositeTemplate,
+    pango,
 };
 use ruma::{
     api::client::{
         directory::{get_room_visibility, set_room_visibility},
-        discovery::get_capabilities::Capabilities,
-        room::{upgrade_room, Visibility},
+        discovery::get_capabilities::v3::Capabilities,
+        room::{Visibility, upgrade_room},
     },
     events::{
+        StateEventType,
         room::{
             guest_access::{GuestAccess, RoomGuestAccessEventContent},
             history_visibility::RoomHistoryVisibilityEventContent,
             power_levels::PowerLevelAction,
         },
-        StateEventType,
     },
 };
 use tracing::error;
 
-use super::{room_upgrade_dialog::confirm_room_upgrade, MemberRow, MembershipLists, RoomDetails};
+use super::{MemberRow, RoomDetails, UpgradeDialog, UpgradeInfo};
 use crate::{
+    Window,
     components::{
-        ButtonCountRow, CheckLoadingRow, ComboLoadingRow, CopyableRow, LoadingButton,
+        Avatar, ButtonCountRow, CheckLoadingRow, ComboLoadingRow, CopyableRow, LoadingButton,
         SwitchLoadingRow,
     },
     gettext_f,
     prelude::*,
     session::model::{
-        HistoryVisibilityValue, JoinRuleValue, Member, NotificationsRoomSetting, Room, RoomCategory,
+        HistoryVisibilityValue, Member, MemberList, MembershipListKind, NotificationsRoomSetting,
+        Room, RoomCategory,
     },
     spawn, spawn_tokio, toast,
-    utils::{expression, matrix::MatrixIdUri, BoundObjectWeakRef, TemplateCallbacks},
-    Window,
+    utils::{BoundObjectWeakRef, TemplateCallbacks, expression, matrix::MatrixIdUri},
 };
 
 mod imp {
@@ -54,6 +55,8 @@ mod imp {
     )]
     #[properties(wrapper_type = super::GeneralPage)]
     pub struct GeneralPage {
+        #[template_child]
+        avatar: TemplateChild<Avatar>,
         #[template_child]
         room_topic: TemplateChild<gtk::Label>,
         #[template_child]
@@ -87,7 +90,7 @@ mod imp {
         canonical_alias_row: RefCell<Option<CopyableRow>>,
         alt_aliases_rows: RefCell<Vec<CopyableRow>>,
         #[template_child]
-        join_rule: TemplateChild<ComboLoadingRow>,
+        join_rule: TemplateChild<ButtonCountRow>,
         #[template_child]
         guest_access: TemplateChild<SwitchLoadingRow>,
         #[template_child]
@@ -103,9 +106,9 @@ mod imp {
         /// The presented room.
         #[property(get, set = Self::set_room, construct_only)]
         room: BoundObjectWeakRef<Room>,
-        /// The lists of members filtered by membership for the room.
-        #[property(get, set = Self::set_membership_lists, construct_only)]
-        membership_lists: glib::WeakRef<MembershipLists>,
+        /// The lists of members in the room.
+        #[property(get, set = Self::set_members, construct_only)]
+        members: glib::WeakRef<MemberList>,
         /// The notifications setting for the room.
         #[property(get = Self::notifications_setting, set = Self::set_notifications_setting, explicit_notify, builder(NotificationsRoomSetting::default()))]
         notifications_setting: PhantomData<NotificationsRoomSetting>,
@@ -115,6 +118,9 @@ mod imp {
         /// Whether the room is published in the directory.
         #[property(get)]
         is_published: Cell<bool>,
+        capabilities: RefCell<Capabilities>,
+        upgrade_info: RefCell<Option<UpgradeInfo>>,
+        direct_members_list_has_bound_model: Cell<bool>,
         expr_watch: RefCell<Option<gtk::ExpressionWatch>>,
         notifications_settings_handlers: RefCell<Vec<glib::SignalHandlerId>>,
         membership_handler: RefCell<Option<glib::SignalHandlerId>>,
@@ -122,8 +128,6 @@ mod imp {
         canonical_alias_handler: RefCell<Option<glib::SignalHandlerId>>,
         alt_aliases_handler: RefCell<Option<glib::SignalHandlerId>>,
         join_rule_handler: RefCell<Option<glib::SignalHandlerId>>,
-        capabilities: RefCell<Capabilities>,
-        direct_members_list_has_bound_model: Cell<bool>,
     }
 
     #[glib::object_subclass]
@@ -316,16 +320,16 @@ mod imp {
             obj.notify_room();
 
             if let Some(session) = room.session() {
-                let settings = session.notifications().settings();
+                let notifications_settings = session.notifications().settings();
                 let notifications_settings_handlers = vec![
-                    settings.connect_account_enabled_notify(clone!(
+                    notifications_settings.connect_account_enabled_notify(clone!(
                         #[weak(rename_to = imp)]
                         self,
                         move |_| {
                             imp.update_notifications();
                         }
                     )),
-                    settings.connect_session_enabled_notify(clone!(
+                    notifications_settings.connect_session_enabled_notify(clone!(
                         #[weak(rename_to = imp)]
                         self,
                         move |_| {
@@ -362,9 +366,9 @@ mod imp {
             self.load_capabilities();
         }
 
-        /// Set the lists of members filtered by membership for the room.
-        fn set_membership_lists(&self, membership_lists: &MembershipLists) {
-            self.membership_lists.set(Some(membership_lists));
+        /// Set the lists of members in the room.
+        fn set_members(&self, members: &MemberList) {
+            self.members.set(Some(members));
             self.update_members();
         }
 
@@ -399,7 +403,7 @@ mod imp {
                     self,
                     async move {
                         let handle = spawn_tokio!(async move { client.get_capabilities().await });
-                        match handle.await.unwrap() {
+                        match handle.await.expect("task was not aborted") {
                             Ok(capabilities) => {
                                 imp.capabilities.replace(capabilities);
                             }
@@ -408,6 +412,8 @@ mod imp {
                                 imp.capabilities.take();
                             }
                         }
+
+                        imp.update_upgrade_info();
                     }
                 )
             );
@@ -445,11 +451,12 @@ mod imp {
             let Some(room) = self.room.obj() else {
                 return;
             };
-            let Some(membership_lists) = self.membership_lists.upgrade() else {
+            let Some(members) = self.members.upgrade() else {
                 return;
             };
 
-            let joined_members_count = membership_lists.joined().n_items();
+            let joined_members = members.membership_list(MembershipListKind::Join);
+            let joined_members_count = joined_members.n_items();
 
             // When the room is direct there should only be 2 members in most cases, but use
             // the members count to make sure we do not show a list that is too long.
@@ -462,7 +469,7 @@ mod imp {
                 // widgets in the background.
                 if !self.direct_members_list_has_bound_model.get() {
                     self.direct_members_list
-                        .bind_model(Some(&membership_lists.joined()), |item| {
+                        .bind_model(Some(&joined_members), |item| {
                             let member = item
                                 .downcast_ref::<Member>()
                                 .expect("joined members list contains members");
@@ -523,9 +530,8 @@ mod imp {
         fn disconnect_all(&self) {
             if let Some(room) = self.room.obj() {
                 if let Some(session) = room.session() {
-                    let settings = session.notifications().settings();
                     for handler in self.notifications_settings_handlers.take() {
-                        settings.disconnect(handler);
+                        session.notifications().settings().disconnect(handler);
                     }
                 }
 
@@ -757,53 +763,13 @@ mod imp {
                 return;
             };
 
-            let row = &self.join_rule;
-            row.set_is_loading(false);
-
-            let permissions = room.permissions();
-            let join_rule = room.join_rule();
-
-            let is_supported_join_rule = matches!(
-                join_rule.value(),
-                JoinRuleValue::Public | JoinRuleValue::Invite
-            ) && !join_rule.can_knock();
-            let can_change = permissions
+            let join_rule_can_be_edited = room.join_rule().value().can_be_edited();
+            let can_change = room
+                .permissions()
                 .is_allowed_to(PowerLevelAction::SendState(StateEventType::RoomJoinRules));
 
-            row.set_read_only(!is_supported_join_rule || !can_change);
-            row.set_selected_string(Some(join_rule.display_name()));
-        }
-
-        /// Set the join rule of the room.
-        #[template_callback]
-        async fn set_join_rule(&self) {
-            let Some(room) = self.room.obj() else {
-                return;
-            };
-            let join_rule = room.join_rule();
-
-            let row = &self.join_rule;
-
-            let value = match row.selected() {
-                0 => JoinRuleValue::Invite,
-                1 => JoinRuleValue::Public,
-                _ => {
-                    return;
-                }
-            };
-
-            if join_rule.value() == value {
-                // Nothing to do.
-                return;
-            }
-
-            row.set_is_loading(true);
-            row.set_read_only(true);
-
-            if join_rule.set_value(value).await.is_err() {
-                toast!(self.obj(), gettext("Could not change who can join"));
-                self.update_join_rule();
-            }
+            self.join_rule
+                .set_activatable(join_rule_can_be_edited && can_change);
         }
 
         /// Update the guest access row.
@@ -987,7 +953,7 @@ mod imp {
             row.set_read_only(!is_supported || !can_change);
         }
 
-        /// Set the history_visibility of the room.
+        /// Set the history visibility of the room.
         #[template_callback]
         async fn set_history_visibility(&self) {
             let Some(room) = self.room.obj() else {
@@ -1084,6 +1050,38 @@ mod imp {
             }
         }
 
+        /// Update the room upgrade info.
+        fn update_upgrade_info(&self) {
+            let Some(room) = self.room.obj() else {
+                return;
+            };
+
+            let room_info = room.matrix_room().clone_info();
+
+            let upgrade_info = room_info.create().map(|create_content| {
+                let privileged_creators = room_info
+                    .room_version_rules_or_default()
+                    .authorization
+                    .explicitly_privilege_room_creators
+                    .then(|| room_info.creators())
+                    .flatten();
+
+                UpgradeInfo::new()
+                    .with_room_versions(
+                        &create_content.room_version,
+                        &self.capabilities.borrow().room_versions,
+                    )
+                    .with_privileged_creators(
+                        room.own_member().user_id(),
+                        &privileged_creators.unwrap_or_default(),
+                    )
+            });
+
+            self.upgrade_info.replace(upgrade_info);
+
+            self.update_upgrade_button();
+        }
+
         /// Update the room upgrade button.
         fn update_upgrade_button(&self) {
             let Some(room) = self.room.obj() else {
@@ -1093,7 +1091,8 @@ mod imp {
             let can_upgrade = !room.is_tombstoned()
                 && room
                     .permissions()
-                    .is_allowed_to(PowerLevelAction::SendState(StateEventType::RoomTombstone));
+                    .is_allowed_to(PowerLevelAction::SendState(StateEventType::RoomTombstone))
+                && self.upgrade_info.borrow().is_some();
             self.upgrade_button.set_visible(can_upgrade);
         }
 
@@ -1120,13 +1119,16 @@ mod imp {
             let Some(room) = self.room.obj() else {
                 return;
             };
+            let Some(upgrade_info) = self.upgrade_info.borrow().clone() else {
+                return;
+            };
 
             let obj = self.obj();
-            // TODO: Hide upgrade button if room already upgraded?
             self.upgrade_button.set_is_loading(true);
-            let room_versions_capability = self.capabilities.borrow().room_versions.clone();
 
-            let Some(new_version) = confirm_room_upgrade(room_versions_capability, &*obj).await
+            let Some(new_version) = UpgradeDialog::new()
+                .confirm_upgrade(&upgrade_info, &*obj)
+                .await
             else {
                 self.upgrade_button.set_is_loading(false);
                 return;
@@ -1170,10 +1172,10 @@ glib::wrapper! {
 }
 
 impl GeneralPage {
-    pub fn new(room: &Room, membership_lists: &MembershipLists) -> Self {
+    pub fn new(room: &Room, members: &MemberList) -> Self {
         glib::Object::builder()
             .property("room", room)
-            .property("membership-lists", membership_lists)
+            .property("members", members)
             .build()
     }
 
