@@ -1,4 +1,4 @@
-use gtk::{gdk, glib, glib::clone, prelude::*, subclass::prelude::*, CompositeTemplate};
+use gtk::{CompositeTemplate, gdk, glib, glib::clone, prelude::*, subclass::prelude::*};
 use ruma::api::client::media::get_content_thumbnail::v3::Method;
 
 use super::{HistoryViewerEvent, VisualMediaHistoryViewer};
@@ -9,8 +9,8 @@ use crate::{
         key_bindings,
         matrix::VisualMediaMessage,
         media::{
-            image::{Blurhash, ImageRequestPriority, ThumbnailSettings},
             FrameDimensions,
+            image::{Blurhash, ImageRequestPriority, ThumbnailSettings},
         },
     },
 };
@@ -160,7 +160,10 @@ mod imp {
                 return;
             };
 
-            if session.settings().should_room_show_media_previews(&room) {
+            if session
+                .global_account_data()
+                .should_room_show_media_previews(&room)
+            {
                 spawn!(
                     glib::Priority::LOW,
                     clone!(
@@ -224,7 +227,7 @@ mod imp {
                 return;
             };
 
-            media_history_viewer.show_media(&obj);
+            media_history_viewer.show_media_viewer(&obj);
         }
     }
 }
