@@ -1,5 +1,5 @@
 use adw::subclass::prelude::BinImpl;
-use gtk::{glib, prelude::*, subclass::prelude::*, CompositeTemplate};
+use gtk::{CompositeTemplate, glib, prelude::*, subclass::prelude::*};
 
 use crate::{session::model::Member, utils::bool_to_accessible_tristate};
 
