@@ -2,7 +2,7 @@ use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::gettext;
 use gtk::{gdk, glib, glib::clone};
 use matrix_sdk_ui::timeline::{MsgLikeKind, TimelineDetails, TimelineItemContent};
-use ruma::{events::room::message::MessageType, OwnedEventId, OwnedTransactionId};
+use ruma::{OwnedEventId, OwnedTransactionId, events::room::message::MessageType};
 use tracing::{error, warn};
 
 use super::{
@@ -56,7 +56,7 @@ mod imp {
         #[property(get = Self::texture)]
         texture: PhantomData<Option<gdk::Texture>>,
         /// The widget with the visual media content of the event, if any.
-        pub(super) visual_media_widget: glib::WeakRef<MessageVisualMedia>,
+        visual_media_widget: glib::WeakRef<MessageVisualMedia>,
     }
 
     #[glib::object_subclass]
@@ -152,11 +152,6 @@ impl MessageContent {
         glib::Object::new()
     }
 
-    /// The widget with the visual media content of the event, if any.
-    pub(crate) fn visual_media_widget(&self) -> Option<MessageVisualMedia> {
-        self.imp().visual_media_widget.upgrade()
-    }
-
     /// Update this widget to present the given `Event`.
     pub(crate) fn update_for_event(&self, event: &Event) {
         let detect_at_room = event.can_contain_at_room() && event.sender().can_notify_room();
@@ -193,16 +188,18 @@ impl MessageContent {
                         let replied_to_sender = event
                             .room()
                             .get_or_create_members()
-                            .get_or_create(replied_to_event.sender().to_owned());
-                        let replied_to_content = replied_to_event.content();
-                        let replied_to_detect_at_room = replied_to_content.can_contain_at_room()
-                            && replied_to_sender.can_notify_room();
+                            .get_or_create(replied_to_event.sender);
+                        let replied_to_detect_at_room =
+                            replied_to_event.content.can_contain_at_room()
+                                && replied_to_sender.can_notify_room();
 
                         let reply = MessageReply::new();
-                        reply.set_show_related_content_header(replied_to_content.can_show_header());
+                        reply.set_show_related_content_header(
+                            replied_to_event.content.can_show_header(),
+                        );
                         reply.set_related_content_sender(replied_to_sender.upcast_ref());
                         reply.related_content().build_content(
-                            replied_to_content.clone(),
+                            replied_to_event.content,
                             ContentFormat::Compact,
                             &replied_to_sender,
                             replied_to_detect_at_room,
@@ -279,6 +276,7 @@ trait MessageContentContainer: ChildPropertyExt {
     ) {
         let room = sender.room();
 
+        #[allow(clippy::match_wildcard_for_single_variants)]
         match content {
             TimelineItemContent::MsgLike(msg_like) => match msg_like.kind {
                 MsgLikeKind::Message(message) => {
@@ -311,6 +309,10 @@ trait MessageContentContainer: ChildPropertyExt {
                     let child = self.child_or_default::<MessageText>();
                     child.with_plain_text(gettext("Could not decrypt this message, decryption will be retried once the keys are available."), format);
                 }
+                MsgLikeKind::Redacted => {
+                    let child = self.child_or_default::<MessageText>();
+                    child.with_plain_text(gettext("This message was removed."), format);
+                }
                 msg_like_kind => {
                     warn!("Unsupported message-like event content: {msg_like_kind:?}");
                     self.build_unsupported_content(format);
@@ -412,13 +414,7 @@ trait MessageContentContainer: ChildPropertyExt {
         if let Some((caption, formatted_caption)) = media_message.caption() {
             let caption_widget = self.child_or_default::<MessageCaption>();
 
-            caption_widget.set_caption(
-                caption.to_owned(),
-                formatted_caption.cloned(),
-                room,
-                format,
-                detect_at_room,
-            );
+            caption_widget.set_caption(caption, formatted_caption, room, format, detect_at_room);
 
             caption_widget.build_media_content(media_message, format, room, cache_key);
         } else {
@@ -490,7 +486,7 @@ pub(crate) struct MessageCacheKey {
     ///
     /// Local echo that was sent and remote echo should have the same event ID,
     /// so we do not need to reload the message if it did not change.
-    event_id: Option<OwnedEventId>,
+    pub(crate) event_id: Option<OwnedEventId>,
     /// Whether the message is edited.
     ///
     /// The message must be reloaded when it was edited.
