@@ -1,6 +1,5 @@
 use adw::{prelude::*, subclass::prelude::*};
-use gtk::{gdk, glib, glib::clone, CompositeTemplate};
-use tracing::warn;
+use gtk::{CompositeTemplate, gdk, glib, glib::clone};
 
 mod audio;
 mod caption;
@@ -11,19 +10,22 @@ mod message_state_stack;
 mod reaction;
 mod reaction_list;
 mod reply;
+mod sender_name;
 mod text;
 mod visual_media;
 
 pub use self::content::{ContentFormat, MessageContent};
-use self::{message_state_stack::MessageStateStack, reaction_list::MessageReactionList};
+use self::{
+    message_state_stack::MessageStateStack, reaction_list::MessageReactionList,
+    sender_name::MessageSenderName,
+};
 use super::{ReadReceiptsList, SenderAvatar};
 use crate::{
-    gettext_f,
+    Application, gettext_f,
     prelude::*,
     session::model::{Event, EventHeaderState},
     system_settings::ClockFormat,
     utils::BoundObject,
-    Application, Window,
 };
 
 mod imp {
@@ -44,7 +46,7 @@ mod imp {
         #[template_child]
         header: TemplateChild<gtk::Box>,
         #[template_child]
-        display_name: TemplateChild<gtk::Label>,
+        display_name: TemplateChild<MessageSenderName>,
         #[template_child]
         timestamp: TemplateChild<gtk::Label>,
         #[template_child]
@@ -55,7 +57,7 @@ mod imp {
         reactions: TemplateChild<MessageReactionList>,
         #[template_child]
         read_receipts: TemplateChild<ReadReceiptsList>,
-        bindings: RefCell<Vec<glib::Binding>>,
+        binding: RefCell<Option<glib::Binding>>,
         system_settings_handler: RefCell<Option<glib::SignalHandlerId>>,
         /// The event that is presented.
         #[property(get, set = Self::set_event, explicit_notify)]
@@ -74,10 +76,6 @@ mod imp {
 
         fn class_init(klass: &mut Self::Class) {
             Self::bind_template(klass);
-
-            klass.install_action("message-row.show-media", None, |obj, _, _| {
-                obj.imp().show_media();
-            });
         }
 
         fn instance_init(obj: &InitializingObject<Self>) {
@@ -122,7 +120,7 @@ mod imp {
         }
 
         fn dispose(&self) {
-            for binding in self.bindings.take() {
+            if let Some(binding) = self.binding.take() {
                 binding.unbind();
             }
 
@@ -142,26 +140,20 @@ mod imp {
 
             // Remove signals and bindings from the previous event.
             self.event.disconnect_signals();
-            while let Some(binding) = self.bindings.borrow_mut().pop() {
+            if let Some(binding) = self.binding.take() {
                 binding.unbind();
             }
 
-            self.avatar.set_sender(Some(event.sender()));
-
-            let display_name_binding = event
-                .sender()
-                .bind_property("disambiguated-name", &*self.display_name, "label")
-                .sync_create()
-                .build();
+            let sender = event.sender();
+            self.avatar.set_sender(Some(sender.clone()));
+            self.display_name.set_sender(Some(sender));
 
             let state_binding = event
                 .bind_property("state", &*self.message_state, "state")
                 .sync_create()
                 .build();
 
-            self.bindings
-                .borrow_mut()
-                .append(&mut vec![display_name_binding, state_binding]);
+            self.binding.replace(Some(state_binding));
 
             let header_state_handler = event.connect_header_state_notify(clone!(
                 #[weak(rename_to = imp)]
@@ -241,7 +233,7 @@ mod imp {
 
             let clock_format = Application::default().system_settings().clock_format();
             let time = if clock_format == ClockFormat::TwelveHours {
-                datetime.format("%I∶%M %p").unwrap()
+                datetime.format("%I:%M %p").unwrap()
             } else {
                 datetime.format("%R").unwrap()
             };
@@ -266,25 +258,6 @@ mod imp {
         pub(super) fn texture(&self) -> Option<gdk::Texture> {
             self.content.texture()
         }
-
-        /// Open the media viewer with the media content of this row.
-        fn show_media(&self) {
-            let Some(window) = self.obj().root().and_downcast::<Window>() else {
-                return;
-            };
-            let Some(event) = self.event.obj() else {
-                return;
-            };
-
-            let Some(visual_media_widget) = self.content.visual_media_widget() else {
-                warn!("Trying to show media of a non-media message");
-                return;
-            };
-
-            window
-                .session_view()
-                .show_media(&event, &visual_media_widget);
-        }
     }
 }
 
