@@ -1,23 +1,23 @@
 use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::gettext;
-use gtk::{gdk, glib, glib::clone, CompositeTemplate};
+use gtk::{CompositeTemplate, gdk, glib, glib::clone};
 use ruma::api::client::media::get_content_thumbnail::v3::Method;
-use tracing::{error, warn};
+use tracing::warn;
 
-use super::{content::MessageCacheKey, ContentFormat};
+use super::{ContentFormat, content::MessageCacheKey};
 use crate::{
+    Window,
     components::{AnimatedImagePaintable, VideoPlayer},
     gettext_f,
     session::model::Room,
     spawn,
     utils::{
-        key_bindings,
+        CountedRef, File, LoadingState, TemplateCallbacks, key_bindings,
         matrix::{VisualMediaMessage, VisualMediaType},
         media::{
-            image::{ImageRequestPriority, ThumbnailSettings, THUMBNAIL_MAX_DIMENSIONS},
             FrameDimensions,
+            image::{ImageRequestPriority, THUMBNAIL_MAX_DIMENSIONS, ThumbnailSettings},
         },
-        CountedRef, File, LoadingState, TemplateCallbacks,
     },
 };
 
@@ -71,7 +71,7 @@ mod imp {
         /// The room where the message was sent.
         room: glib::WeakRef<Room>,
         join_rule_handler: RefCell<Option<glib::SignalHandlerId>>,
-        session_settings_handler: RefCell<Option<glib::SignalHandlerId>>,
+        global_account_data_handler: RefCell<Option<glib::SignalHandlerId>>,
         /// The visual media message to display.
         media_message: RefCell<Option<VisualMediaMessage>>,
         /// The cache key for the current media message.
@@ -298,7 +298,9 @@ mod imp {
             self.spinner.set_visible(state == LoadingState::Loading);
             self.hide_preview_button.set_visible(
                 state == LoadingState::Ready
-                    && !session.settings().should_room_show_media_previews(&room),
+                    && !session
+                        .global_account_data()
+                        .should_room_show_media_previews(&room),
             );
             self.error.set_visible(state == LoadingState::Error);
 
@@ -473,7 +475,7 @@ mod imp {
                 return;
             };
 
-            let join_rule_handler = room.join_rule().connect_anyone_can_join_notify(clone!(
+            let join_rule_handler = room.join_rule().connect_value_notify(clone!(
                 #[weak(rename_to = imp)]
                 self,
                 move |_| {
@@ -482,8 +484,8 @@ mod imp {
             ));
             self.join_rule_handler.replace(Some(join_rule_handler));
 
-            let session_settings_handler = session
-                .settings()
+            let global_account_data_handler = session
+                .global_account_data()
                 .connect_media_previews_enabled_changed(clone!(
                     #[weak(rename_to = imp)]
                     self,
@@ -491,8 +493,8 @@ mod imp {
                         imp.update_media();
                     }
                 ));
-            self.session_settings_handler
-                .replace(Some(session_settings_handler));
+            self.global_account_data_handler
+                .replace(Some(global_account_data_handler));
 
             self.room.set(Some(room));
 
@@ -595,7 +597,10 @@ mod imp {
                 return;
             };
 
-            if session.settings().should_room_show_media_previews(&room) {
+            if session
+                .global_account_data()
+                .should_room_show_media_previews(&room)
+            {
                 // Only load the media if it was not loaded before.
                 if self.state.get() == LoadingState::Initial {
                     self.show_media();
@@ -796,9 +801,9 @@ mod imp {
                     room.join_rule().disconnect(handler);
                 }
 
-                if let Some(handler) = self.session_settings_handler.take() {
+                if let Some(handler) = self.global_account_data_handler.take() {
                     if let Some(session) = room.session() {
-                        session.settings().disconnect(handler);
+                        session.global_account_data().disconnect(handler);
                     }
                 }
             }
@@ -808,14 +813,32 @@ mod imp {
         fn activate(&self) {
             if self.state.get() == LoadingState::Initial {
                 self.show_media();
-            } else if self
-                .obj()
-                .activate_action("message-row.show-media", None)
-                .is_err()
-            {
-                error!("Could not activate `message-row.show-media` action");
+            } else {
+                self.show_media_viewer();
             }
         }
+
+        /// Open the media viewer with the media content of this row.
+        fn show_media_viewer(&self) {
+            let Some(room) = self.room.upgrade() else {
+                return;
+            };
+            let Some(media_message) = self.media_message.borrow().clone() else {
+                return;
+            };
+
+            let obj = self.obj();
+
+            let Some(window) = obj.root().and_downcast::<Window>() else {
+                return;
+            };
+
+            let event_id = self.cache_key.borrow().event_id.clone();
+
+            window
+                .session_view()
+                .show_media_viewer(&*self.obj(), &room, media_message, event_id);
+        }
     }
 }
 
