@@ -1,11 +1,11 @@
 use gettextrs::gettext;
-use gtk::{gdk, glib, glib::clone, prelude::*, subclass::prelude::*, CompositeTemplate};
+use gtk::{CompositeTemplate, gdk, glib, glib::clone, prelude::*, subclass::prelude::*};
 use pulldown_cmark::{Event, Parser, Tag};
 use secular::normalized_lower_lay_string;
 
 use super::{CompletionMemberList, CompletionRoomList};
 use crate::{
-    components::{Pill, PillSource, PillSourceRow},
+    components::{AvatarImageSafetySetting, Pill, PillSource, PillSourceRow},
     session::{model::Room, view::content::room_history::message_toolbar::MessageToolbar},
     utils::BoundObject,
 };
@@ -140,10 +140,7 @@ mod imp {
     impl CompletionPopover {
         /// Set the current room.
         fn set_room(&self, room: Option<&Room>) {
-            // `RoomHistory` should have a strong reference to the list so we can use
-            // `get_or_create_members()`.
-            self.member_list
-                .set_members(room.map(Room::get_or_create_members));
+            self.member_list.set_room(room);
 
             self.room_list
                 .set_rooms(room.and_then(Room::session).map(|s| s.room_list()));
@@ -651,7 +648,9 @@ mod imp {
 
             buffer.delete(&mut start, &mut end);
 
-            let pill = Pill::new(&source);
+            // We do not need to watch safety settings for mentions, rooms will be watched
+            // automatically.
+            let pill = Pill::new(&source, AvatarImageSafetySetting::None, None);
             self.message_toolbar()
                 .current_composer_state()
                 .add_widget(pill, &mut start);
