@@ -4,25 +4,24 @@ use gtk::{
     prelude::*,
     subclass::prelude::*,
 };
-use matrix_sdk::{deserialized_responses::TimelineEvent, ComposerDraft, ComposerDraftType};
+use matrix_sdk::{ComposerDraft, ComposerDraftType, deserialized_responses::TimelineEvent};
 use matrix_sdk_ui::timeline::Message;
 use ruma::{
-    events::{
-        room::message::{MessageFormat, MessageType, OriginalSyncRoomMessageEvent},
-        AnySyncMessageLikeEvent, AnySyncTimelineEvent, SyncMessageLikeEvent,
-    },
     OwnedEventId, OwnedUserId, RoomOrAliasId, UserId,
+    events::{
+        AnySyncMessageLikeEvent, AnySyncTimelineEvent, SyncMessageLikeEvent,
+        room::message::{MessageFormat, MessageType, OriginalSyncRoomMessageEvent},
+    },
 };
 use sourceview::prelude::*;
 use tracing::{error, warn};
 
 use super::ComposerParser;
 use crate::{
-    components::{Pill, PillSource},
-    prelude::*,
+    components::{AvatarImageSafetySetting, Pill, PillSource},
     session::model::{Event, Member, Room, Timeline},
     spawn, spawn_tokio,
-    utils::matrix::{find_at_room, find_html_mentions, AT_ROOM},
+    utils::matrix::{AT_ROOM, find_at_room, find_html_mentions},
 };
 
 // The duration in seconds we wait for before saving a change.
@@ -218,9 +217,9 @@ mod imp {
             let draft_clone = draft.clone();
             let handle = spawn_tokio!(async move {
                 if let Some(draft) = draft_clone {
-                    matrix_room.save_composer_draft(draft).await
+                    matrix_room.save_composer_draft(draft, None).await
                 } else {
-                    matrix_room.clear_composer_draft().await
+                    matrix_room.clear_composer_draft(None).await
                 }
             });
 
@@ -267,7 +266,7 @@ mod imp {
         /// Restore the state from the persisted draft.
         pub(super) async fn restore_draft(&self, timeline: &Timeline) {
             let matrix_room = timeline.room().matrix_room().clone();
-            let handle = spawn_tokio!(async move { matrix_room.load_composer_draft().await });
+            let handle = spawn_tokio!(async move { matrix_room.load_composer_draft(None).await });
 
             match handle.await.expect("task was not aborted") {
                 Ok(Some(draft)) => self.restore_from_draft(timeline, draft).await,
@@ -312,7 +311,10 @@ mod imp {
 
                 match DraftMention::new(&room, &text[content_start..content_end]) {
                     DraftMention::Source(source) => {
-                        self.add_widget(source.to_pill().upcast(), &mut end_iter);
+                        // We do not need to watch safety settings for mentions, rooms will be
+                        // watched automatically.
+                        let pill = Pill::new(&source, AvatarImageSafetySetting::None, None);
+                        self.add_widget(pill.upcast(), &mut end_iter);
                     }
                     DraftMention::Text(s) => {
                         self.buffer.insert(&mut end_iter, s);
@@ -387,7 +389,9 @@ mod imp {
             let can_contain_at_room = message.mentions().is_none_or(|m| m.room);
             if room.permissions().can_notify_room() && can_contain_at_room {
                 if let Some(start) = find_at_room(&text) {
-                    let pill = room.at_room().to_pill();
+                    // We do not need to watch safety settings for at-room mentions, our own member
+                    // is in the room.
+                    let pill = Pill::new(&room.at_room(), AvatarImageSafetySetting::None, None);
                     let end = start + AT_ROOM.len();
                     mentions.push(DetectedMention { pill, start, end });
 
@@ -517,7 +521,7 @@ impl ComposerState {
 #[derive(Debug, Clone)]
 pub(crate) enum RelationInfo {
     /// Send a reply to the given event.
-    Reply(MessageEventSource),
+    Reply(Box<MessageEventSource>),
 
     /// Send an edit to the event with the given ID.
     Edit(OwnedEventId),
@@ -551,7 +555,7 @@ impl RelationInfo {
                     return None;
                 };
 
-                Some(RelationInfo::Reply(message_event))
+                Some(RelationInfo::Reply(message_event.into()))
             }
             ComposerDraftType::Edit { event_id } => Some(RelationInfo::Edit(event_id)),
         }
@@ -648,7 +652,7 @@ struct DetectedMention {
 #[derive(Debug, Clone)]
 pub(crate) enum MessageEventSource {
     /// An original event.
-    OriginalEvent(OriginalSyncRoomMessageEvent),
+    OriginalEvent(Box<OriginalSyncRoomMessageEvent>),
     /// An [`Event`].
     Event(Event),
 }
@@ -663,7 +667,7 @@ impl MessageEventSource {
         match event {
             AnySyncTimelineEvent::MessageLike(AnySyncMessageLikeEvent::RoomMessage(
                 SyncMessageLikeEvent::Original(message_event),
-            )) => Some(Self::OriginalEvent(message_event)),
+            )) => Some(Self::OriginalEvent(message_event.into())),
             _ => None,
         }
     }
