@@ -2,12 +2,13 @@ use std::time::Duration;
 
 use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::gettext;
-use gtk::{gdk, gio, glib, glib::clone, graphene::Point, CompositeTemplate};
+use gtk::{CompositeTemplate, gdk, gio, glib, glib::clone, graphene::Point};
 use matrix_sdk::ruma::EventId;
 use matrix_sdk_ui::timeline::TimelineEventItemId;
 use ruma::{
-    api::client::receipt::create_receipt::v3::ReceiptType, events::room::message::MessageType,
     OwnedEventId,
+    api::client::receipt::create_receipt::v3::ReceiptType,
+    events::room::{message::MessageType, power_levels::PowerLevelAction},
 };
 use tracing::{error, warn};
 
@@ -37,17 +38,18 @@ use self::{
     typing_row::TypingRow,
     verification_info_bar::VerificationInfoBar,
 };
-use super::{room_details, RoomDetails};
+use super::{RoomDetails, room_details};
 use crate::{
-    components::{confirm_leave_room_dialog, DragOverlay},
+    Window,
+    components::{DragOverlay, confirm_leave_room_dialog},
+    ngettext_f,
     prelude::*,
     session::model::{
-        Event, MemberList, Membership, ReceiptPosition, Room, TargetRoomCategory, Timeline,
-        VirtualItem, VirtualItemKind,
+        Event, MemberList, Membership, MembershipListKind, ReceiptPosition, Room,
+        TargetRoomCategory, Timeline, VirtualItem, VirtualItemKind,
     },
     spawn, toast,
     utils::{BoundObject, GroupingListGroup, GroupingListModel, LoadingState, TemplateCallbacks},
-    Window,
 };
 
 /// The time to wait before considering that scrolling has ended.
@@ -78,6 +80,8 @@ mod imp {
         #[template_child]
         room_menu: TemplateChild<gtk::MenuButton>,
         #[template_child]
+        pending_knocks_banner: TemplateChild<adw::Banner>,
+        #[template_child]
         listview: TemplateChild<gtk::ListView>,
         #[template_child]
         content: TemplateChild<gtk::Widget>,
@@ -96,8 +100,6 @@ mod imp {
         #[template_child]
         stack: TemplateChild<gtk::Stack>,
         #[template_child]
-        tombstoned_banner: TemplateChild<adw::Banner>,
-        #[template_child]
         drag_overlay: TemplateChild<DragOverlay>,
         /// The context menu for rows presenting an [`Event`].
         event_context_menu: OnceCell<EventActionsContextMenu>,
@@ -123,10 +125,12 @@ mod imp {
         grouping_model: OnceCell<GroupingListModel>,
         scroll_timeout: RefCell<Option<glib::SourceId>>,
         read_timeout: RefCell<Option<glib::SourceId>>,
-        room_handlers: RefCell<Vec<glib::SignalHandlerId>>,
-        can_invite_handler: RefCell<Option<glib::SignalHandlerId>>,
+        room_handler: RefCell<Option<glib::SignalHandlerId>>,
+        permissions_handlers: RefCell<Vec<glib::SignalHandlerId>>,
         membership_handler: RefCell<Option<glib::SignalHandlerId>>,
         join_rule_handler: RefCell<Option<glib::SignalHandlerId>>,
+        knock_items_changed_handler: RefCell<Option<glib::SignalHandlerId>>,
+        window_active_handler: RefCell<Option<glib::SignalHandlerId>>,
     }
 
     #[glib::object_subclass]
@@ -135,6 +139,7 @@ mod imp {
         type Type = super::RoomHistory;
         type ParentType = adw::Bin;
 
+        #[allow(clippy::too_many_lines)]
         fn class_init(klass: &mut Self::Class) {
             VerificationInfoBar::ensure_type();
 
@@ -155,10 +160,13 @@ mod imp {
             });
 
             klass.install_action("room-history.details", None, |obj, _, _| {
-                obj.open_room_details(None);
+                obj.imp().open_room_details(room_details::InitialView::None);
             });
             klass.install_action("room-history.invite-members", None, |obj, _, _| {
-                obj.open_room_details(Some(room_details::SubpageName::Invite));
+                obj.imp()
+                    .open_room_details(room_details::InitialView::Subpage(
+                        room_details::SubpageName::Invite,
+                    ));
             });
 
             klass.install_action(
@@ -275,10 +283,38 @@ mod imp {
                         revealer.set_visible(false);
                     }
                 });
+
+            self.obj().connect_root_notify(|obj| {
+                let imp = obj.imp();
+
+                let Some(window) = imp.parent_window() else {
+                    return;
+                };
+
+                let active_handler = window.connect_is_active_notify(clone!(
+                    #[weak]
+                    imp,
+                    move |window| {
+                        if !window.is_active() {
+                            return;
+                        }
+
+                        // When the window becomes active, trigger a read receipt update.
+                        imp.trigger_read_receipts_update();
+                    }
+                ));
+                imp.window_active_handler.replace(Some(active_handler));
+            });
         }
 
         fn dispose(&self) {
             self.disconnect_all();
+
+            if let Some(handler) = self.window_active_handler.take() {
+                if let Some(window) = self.parent_window() {
+                    window.disconnect(handler);
+                }
+            }
         }
     }
 
@@ -290,6 +326,13 @@ mod imp {
                 self.room_title.grab_focus()
             }
         }
+
+        fn map(&self) {
+            self.parent_map();
+
+            // When the room history becomes mapped, trigger a read receipt update.
+            self.trigger_read_receipts_update();
+        }
     }
 
     impl BinImpl for RoomHistory {}
@@ -391,13 +434,15 @@ mod imp {
         /// Disconnect all the signals.
         fn disconnect_all(&self) {
             if let Some(room) = self.room() {
-                for handler in self.room_handlers.take() {
+                if let Some(handler) = self.room_handler.take() {
                     room.disconnect(handler);
                 }
 
-                if let Some(handler) = self.can_invite_handler.take() {
-                    room.permissions().disconnect(handler);
+                let permissions = room.permissions();
+                for handler in self.permissions_handlers.take() {
+                    permissions.disconnect(handler);
                 }
+
                 if let Some(handler) = self.membership_handler.take() {
                     room.own_member().disconnect(handler);
                 }
@@ -406,6 +451,14 @@ mod imp {
                 }
             }
 
+            if let Some(members) = self.room_members.take() {
+                if let Some(handler) = self.knock_items_changed_handler.take() {
+                    members
+                        .membership_list(MembershipListKind::Knock)
+                        .disconnect(handler);
+                }
+            }
+
             self.timeline.disconnect_signals();
         }
 
@@ -429,8 +482,21 @@ mod imp {
 
                 // Keep a strong reference to the members list before changing the model, so all
                 // events use the same list.
-                self.room_members
-                    .replace(Some(room.get_or_create_members()));
+                let room_members = room.get_or_create_members();
+
+                let knock_items_changed_handler = room_members
+                    .membership_list(MembershipListKind::Knock)
+                    .connect_items_changed(clone!(
+                        #[weak(rename_to = imp)]
+                        self,
+                        move |_, _, _, _| {
+                            imp.update_pending_knocks();
+                        }
+                    ));
+                self.knock_items_changed_handler
+                    .replace(Some(knock_items_changed_handler));
+
+                self.room_members.replace(Some(room_members));
 
                 let membership_handler = room.own_member().connect_membership_notify(clone!(
                     #[weak(rename_to = imp)]
@@ -457,7 +523,15 @@ mod imp {
                         imp.update_invite_action();
                     }
                 ));
-                self.can_invite_handler.replace(Some(can_invite_handler));
+                let changed_handler = room.permissions().connect_changed(clone!(
+                    #[weak(rename_to = imp)]
+                    self,
+                    move |_| {
+                        imp.update_pending_knocks();
+                    }
+                ));
+                self.permissions_handlers
+                    .replace(vec![can_invite_handler, changed_handler]);
 
                 let is_direct_handler = room.connect_is_direct_notify(clone!(
                     #[weak(rename_to = imp)]
@@ -466,34 +540,8 @@ mod imp {
                         imp.update_invite_action();
                     }
                 ));
-                let tombstoned_handler = room.connect_is_tombstoned_notify(clone!(
-                    #[weak(rename_to = imp)]
-                    self,
-                    move |_| {
-                        imp.update_tombstoned_banner();
-                    }
-                ));
-                let successor_id_handler = room.connect_successor_id_string_notify(clone!(
-                    #[weak(rename_to = imp)]
-                    self,
-                    move |_| {
-                        imp.update_tombstoned_banner();
-                    }
-                ));
-                let successor_handler = room.connect_successor_notify(clone!(
-                    #[weak(rename_to = imp)]
-                    self,
-                    move |_| {
-                        imp.update_tombstoned_banner();
-                    }
-                ));
 
-                self.room_handlers.replace(vec![
-                    is_direct_handler,
-                    tombstoned_handler,
-                    successor_id_handler,
-                    successor_handler,
-                ]);
+                self.room_handler.replace(Some(is_direct_handler));
 
                 let empty_handler = timeline.connect_is_empty_notify(clone!(
                     #[weak(rename_to = imp)]
@@ -533,8 +581,8 @@ mod imp {
             self.update_view();
             self.load_more_events_if_needed();
             self.update_room_menu();
-            self.update_tombstoned_banner();
             self.update_invite_action();
+            self.update_pending_knocks();
 
             self.obj().notify_timeline();
         }
@@ -800,6 +848,21 @@ mod imp {
             }
         }
 
+        /// The ancestor window of the room history.
+        fn parent_window(&self) -> Option<Window> {
+            self.obj().root().and_downcast()
+        }
+
+        /// Whether the room history is active.
+        ///
+        /// It means that the ancestor window is active and the room history is
+        /// mapped.
+        fn is_active(&self) -> bool {
+            self.parent_window()
+                .is_some_and(|window| window.is_active())
+                && self.obj().is_mapped()
+        }
+
         /// Trigger the process to update read receipts.
         fn trigger_read_receipts_update(&self) {
             let Some(timeline) = self.timeline.obj() else {
@@ -814,6 +877,10 @@ mod imp {
                     source_id.remove();
                 }
 
+                if !self.is_active() {
+                    return;
+                }
+
                 // Only send read receipt when scrolling stopped.
                 self.scroll_timeout
                     .replace(Some(glib::timeout_add_local_once(
@@ -837,6 +904,10 @@ mod imp {
                 source_id.remove();
             }
 
+            if !self.is_active() {
+                return;
+            }
+
             self.read_timeout.replace(Some(glib::timeout_add_local_once(
                 READ_TIMEOUT,
                 clone!(
@@ -866,6 +937,10 @@ mod imp {
         fn update_read_marker(&self) {
             self.read_timeout.take();
 
+            if !self.is_active() {
+                return;
+            }
+
             let Some(position) = self.receipt_position() else {
                 return;
             };
@@ -931,36 +1006,6 @@ mod imp {
             None
         }
 
-        /// Update the tombstoned banner according to the state of the current
-        /// room.
-        fn update_tombstoned_banner(&self) {
-            let banner = &self.tombstoned_banner;
-
-            let Some(room) = self.room() else {
-                banner.set_revealed(false);
-                return;
-            };
-
-            if !room.is_tombstoned() {
-                banner.set_revealed(false);
-                return;
-            }
-
-            if room.successor().is_some() {
-                banner.set_title(&gettext("There is a newer version of this room"));
-                // Translators: This is a verb, as in 'View Room'.
-                banner.set_button_label(Some(&gettext("View")));
-            } else if room.successor_id().is_some() {
-                banner.set_title(&gettext("There is a newer version of this room"));
-                banner.set_button_label(Some(&gettext("Join")));
-            } else {
-                banner.set_title(&gettext("This room was closed"));
-                banner.set_button_label(None);
-            }
-
-            banner.set_revealed(true);
-        }
-
         /// Leave the room.
         async fn leave(&self) {
             let Some(room) = self.room() else {
@@ -1042,42 +1087,39 @@ mod imp {
                 .action_set_enabled("room-history.invite-members", can_invite);
         }
 
-        /// Join or view the successor of the room, if possible.
-        #[template_callback]
-        async fn join_or_view_successor(&self) {
-            let Some(room) = self.room() else {
+        // Update the pending knocks according to the current state.
+        fn update_pending_knocks(&self) {
+            if self.room().is_none_or(|room| {
+                let permissions = room.permissions();
+                !permissions.is_allowed_to(PowerLevelAction::Invite)
+                    && !permissions.is_allowed_to(PowerLevelAction::Kick)
+                    && !permissions.is_allowed_to(PowerLevelAction::Ban)
+            }) {
+                // Our user cannot act on the knock.
+                self.pending_knocks_banner.set_revealed(false);
                 return;
-            };
-            let Some(session) = room.session() else {
+            }
+
+            let Some(members) = self.room_members.borrow().clone() else {
+                self.pending_knocks_banner.set_revealed(false);
                 return;
             };
 
-            if !room.is_joined() || !room.is_tombstoned() {
-                return;
+            let n = members.membership_list(MembershipListKind::Knock).n_items();
+            let reveal = n > 0;
+
+            if reveal {
+                self.pending_knocks_banner.set_title(&ngettext_f(
+                    // Translators: Do NOT translate the content between '{' and '}',
+                    // this is a variable name.
+                    "There is a pending invite request",
+                    "There are {n} pending invite requests",
+                    n,
+                    &[("n", &n.to_string())],
+                ));
             }
-            let obj = self.obj();
 
-            if let Some(successor) = room.successor() {
-                let Some(window) = obj.root().and_downcast::<Window>() else {
-                    return;
-                };
-
-                window.session_view().select_room(successor);
-            } else if let Some(successor_id) = room.successor_id().map(ToOwned::to_owned) {
-                let via = successor_id
-                    .server_name()
-                    .map(ToOwned::to_owned)
-                    .into_iter()
-                    .collect();
-
-                if let Err(error) = session
-                    .room_list()
-                    .join_by_id_or_alias(successor_id.into(), via)
-                    .await
-                {
-                    toast!(obj, error);
-                }
-            }
+            self.pending_knocks_banner.set_revealed(reveal);
         }
 
         /// The context menu for rows presenting an [`Event`].
@@ -1099,6 +1141,26 @@ mod imp {
                 popover
             })
         }
+
+        /// Opens the room details with the given initial view.
+        fn open_room_details(&self, initial_view: room_details::InitialView) {
+            let Some(room) = self.room() else {
+                return;
+            };
+
+            let window =
+                RoomDetails::new(self.obj().root().and_downcast_ref(), &room, initial_view);
+
+            window.present();
+        }
+
+        /// View the list of pending knock requests.
+        #[template_callback]
+        fn view_pending_knocks(&self) {
+            self.open_room_details(room_details::InitialView::Members(
+                MembershipListKind::Knock,
+            ));
+        }
     }
 }
 
@@ -1123,22 +1185,6 @@ impl RoomHistory {
         &self.imp().message_toolbar
     }
 
-    /// Opens the room details.
-    ///
-    /// If `subpage_name` is set, the room details will be opened on the given
-    /// subpage.
-    pub(crate) fn open_room_details(&self, subpage_name: Option<room_details::SubpageName>) {
-        let Some(room) = self.imp().room() else {
-            return;
-        };
-
-        let window = RoomDetails::new(self.root().and_downcast_ref(), &room);
-        if let Some(subpage_name) = subpage_name {
-            window.show_initial_subpage(subpage_name);
-        }
-        window.present();
-    }
-
     /// Enable or disable the mode allowing the room history to stick to the
     /// bottom based on scrollbar position.
     pub(crate) fn enable_sticky_mode(&self, enable: bool) {
