@@ -1,12 +1,18 @@
+use std::slice;
+
 use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::{gettext, ngettext};
-use gtk::{gdk, glib, glib::clone, CompositeTemplate};
-use ruma::{events::room::power_levels::PowerLevelUserAction, OwnedEventId};
+use gtk::{CompositeTemplate, gdk, glib, glib::clone};
+use ruma::{
+    Int, OwnedEventId,
+    events::room::power_levels::{PowerLevelUserAction, UserPowerLevel},
+};
 
 use crate::{
+    Window,
     components::{
-        confirm_mute_room_member_dialog, confirm_room_member_destructive_action_dialog, Avatar,
-        RoomMemberDestructiveAction, UserProfileDialog,
+        Avatar, RoomMemberDestructiveAction, UserProfileDialog, confirm_mute_room_member_dialog,
+        confirm_room_member_destructive_action_dialog,
     },
     gettext_f,
     prelude::*,
@@ -15,8 +21,7 @@ use crate::{
         view::content::RoomHistory,
     },
     toast,
-    utils::{key_bindings, BoundObject},
-    Window,
+    utils::{BoundObject, key_bindings},
 };
 
 mod imp {
@@ -123,10 +128,6 @@ mod imp {
                 obj.imp().kick().await;
             });
 
-            klass.install_action_async("sender-avatar.deny-access", None, |obj, _, _| async move {
-                obj.imp().kick().await;
-            });
-
             klass.install_action_async("sender-avatar.ban", None, |obj, _, _| async move {
                 obj.imp().ban().await;
             });
@@ -202,7 +203,7 @@ mod imp {
 
     #[gtk::template_callbacks]
     impl SenderAvatar {
-        /// Set the list of room members.
+        /// Set the displayed member.
         fn set_sender(&self, sender: Option<Member>) {
             let prev_sender = self.sender.obj();
 
@@ -249,7 +250,7 @@ mod imp {
                     }
                 ));
 
-                let power_level_handler = sender.connect_power_level_notify(clone!(
+                let power_level_handler = sender.connect_power_level_changed(clone!(
                     #[weak(rename_to = imp)]
                     self,
                     move |_| {
@@ -373,13 +374,6 @@ mod imp {
                     && permissions.can_do_to_user(sender_id, PowerLevelUserAction::Kick),
             );
 
-            obj.action_set_enabled(
-                "sender-avatar.deny-access",
-                !is_own_user
-                    && membership == Membership::Knock
-                    && permissions.can_do_to_user(sender_id, PowerLevelUserAction::Kick),
-            );
-
             obj.action_set_enabled(
                 "sender-avatar.ban",
                 !is_own_user
@@ -599,7 +593,6 @@ mod imp {
 
             let label = match membership {
                 Membership::Invite => gettext("Revoking invite…"),
-                Membership::Knock => gettext("Denying access…"),
                 _ => gettext("Kicking user…"),
             };
             toast!(obj, label);
@@ -609,7 +602,6 @@ mod imp {
             if room.kick(&[(user_id, response.reason)]).await.is_err() {
                 let error = match membership {
                     Membership::Invite => gettext("Could not revoke invite of user"),
-                    Membership::Knock => gettext("Could not deny access to user"),
                     _ => gettext("Could not kick user"),
                 };
                 toast!(obj, error);
@@ -621,15 +613,20 @@ mod imp {
             let Some(sender) = self.sender.obj() else {
                 return;
             };
-            let obj = self.obj();
 
-            let old_power_level = sender.power_level();
+            let UserPowerLevel::Int(old_power_level) = sender.power_level() else {
+                // We cannot mute someone with an infinite power level.
+                return;
+            };
+
+            let old_power_level = i64::from(old_power_level);
+            let obj = self.obj();
             let permissions = sender.room().permissions();
 
             // Warn if user is muted but was not before.
             let mute_power_level = permissions.mute_power_level();
             let mute = old_power_level > mute_power_level;
-            if mute && !confirm_mute_room_member_dialog(&sender, &*obj).await {
+            if mute && !confirm_mute_room_member_dialog(slice::from_ref(&sender), &*obj).await {
                 return;
             }
 
@@ -646,7 +643,7 @@ mod imp {
             toast!(obj, text);
 
             let text = if permissions
-                .set_user_power_level(user_id, new_power_level)
+                .set_user_power_level(user_id, Int::new_saturating(new_power_level))
                 .await
                 .is_ok()
             {
