@@ -3,7 +3,6 @@ mod creation;
 mod group_item_row;
 mod group_row;
 mod row;
-mod tombstone;
 
-use self::{content::*, creation::*, group_item_row::*, tombstone::*};
+use self::{content::*, creation::*, group_item_row::*};
 pub(super) use self::{group_row::*, row::*};
