@@ -1,9 +1,8 @@
 use adw::{prelude::*, subclass::prelude::*};
 use gettextrs::gettext;
 use gtk::{
-    gio,
+    CompositeTemplate, ListScrollFlags, gio,
     glib::{self, clone, closure_local},
-    CompositeTemplate, ListScrollFlags,
 };
 use tracing::error;
 
@@ -17,15 +16,15 @@ use self::{
     icon_item_row::SidebarIconItemRow, room_row::SidebarRoomRow, row::SidebarRow,
     section_row::SidebarSectionRow, verification_row::SidebarVerificationRow,
 };
-use super::{account_settings::AccountSettingsSubpage, AccountSettings};
+use super::{AccountSettings, account_settings::AccountSettingsSubpage};
 use crate::{
     account_switcher::AccountSwitcherButton,
     components::OfflineBanner,
     session::model::{
-        CryptoIdentityState, RecoveryState, RoomCategory, Selection, Session,
-        SessionVerificationState, SidebarListModel, SidebarSection, TargetRoomCategory, User,
+        CryptoIdentityState, RecoveryState, RoomCategory, Session, SessionVerificationState,
+        SidebarListModel, SidebarSection, TargetRoomCategory, User,
     },
-    utils::expression,
+    utils::{FixedSelection, expression},
 };
 
 mod imp {
@@ -45,6 +44,8 @@ mod imp {
         #[template_child]
         pub(super) header_bar: TemplateChild<adw::HeaderBar>,
         #[template_child]
+        account_switcher_button: TemplateChild<AccountSwitcherButton>,
+        #[template_child]
         security_banner: TemplateChild<adw::Banner>,
         #[template_child]
         scrolled_window: TemplateChild<gtk::ScrolledWindow>,
@@ -79,7 +80,6 @@ mod imp {
         type ParentType = adw::NavigationPage;
 
         fn class_init(klass: &mut Self::Class) {
-            AccountSwitcherButton::ensure_type();
             OfflineBanner::ensure_type();
 
             Self::bind_template(klass);
@@ -126,7 +126,7 @@ mod imp {
             self.listview.set_factory(Some(&factory));
 
             self.listview.connect_activate(move |listview, pos| {
-                let Some(model) = listview.model().and_downcast::<Selection>() else {
+                let Some(model) = listview.model().and_downcast::<FixedSelection>() else {
                     return;
                 };
                 let Some(item) = model.item(pos) else {
@@ -171,7 +171,16 @@ mod imp {
         }
     }
 
-    impl WidgetImpl for Sidebar {}
+    impl WidgetImpl for Sidebar {
+        fn grab_focus(&self) -> bool {
+            if self.listview.grab_focus() {
+                true
+            } else {
+                self.account_switcher_button.grab_focus()
+            }
+        }
+    }
+
     impl NavigationPageImpl for Sidebar {}
 
     #[gtk::template_callbacks]
@@ -389,8 +398,8 @@ mod imp {
 
             let dialog = AccountSettings::new(&session);
 
-            // Show the security tab if the user uses the back button.
-            dialog.set_visible_page_name("security");
+            // Show the encryption tab if the user uses the back button.
+            dialog.show_encryption_tab();
 
             let security = session.security();
             let crypto_identity_state = security.crypto_identity_state();
