@@ -1,8 +1,9 @@
 use adw::{prelude::*, subclass::prelude::*};
-use gtk::{gdk, glib, glib::clone, CompositeTemplate};
+use gtk::{CompositeTemplate, gdk, glib, glib::clone};
 
 use super::SidebarRow;
 use crate::{
+    components::Avatar,
     i18n::{gettext_f, ngettext_f},
     prelude::*,
     session::model::{HighlightFlags, Room, RoomCategory},
@@ -24,6 +25,8 @@ mod imp {
         #[property(get, set = Self::set_room, explicit_notify, nullable)]
         room: BoundObject<Room>,
         #[template_child]
+        avatar: TemplateChild<Avatar>,
+        #[template_child]
         display_name_box: TemplateChild<gtk::Box>,
         #[template_child]
         display_name: TemplateChild<gtk::Label>,
@@ -95,7 +98,6 @@ mod imp {
             }
 
             self.room.disconnect_signals();
-            self.display_name.remove_css_class("dimmed");
 
             if let Some(room) = room {
                 let highlight_handler = room.connect_highlight_notify(clone!(
@@ -126,10 +128,13 @@ mod imp {
                         imp.update_accessibility_label();
                     }
                 ));
-
-                if room.category() == RoomCategory::Left {
-                    self.display_name.add_css_class("dimmed");
-                }
+                let category_handler = room.connect_category_notify(clone!(
+                    #[weak(rename_to = imp)]
+                    self,
+                    move |_| {
+                        imp.update_display_name();
+                    }
+                ));
 
                 self.room.set(
                     room,
@@ -138,17 +143,32 @@ mod imp {
                         direct_handler,
                         name_handler,
                         notifications_count_handler,
+                        category_handler,
                     ],
                 );
 
                 self.update_accessibility_label();
             }
 
+            self.update_display_name();
             self.update_highlight();
             self.update_direct_icon();
             self.obj().notify_room();
         }
 
+        /// Update the display name of the room according to the current state.
+        fn update_display_name(&self) {
+            let Some(room) = self.room.obj() else {
+                return;
+            };
+
+            if matches!(room.category(), RoomCategory::Left) {
+                self.display_name.add_css_class("dimmed");
+            } else {
+                self.display_name.remove_css_class("dimmed");
+            }
+        }
+
         /// Update how this row is highlighted according to the current state.
         fn update_highlight(&self) {
             if let Some(room) = self.room.obj() {
