@@ -8,7 +8,7 @@ use super::{
     Sidebar, SidebarIconItemRow, SidebarRoomRow, SidebarSectionRow, SidebarVerificationRow,
 };
 use crate::{
-    components::{confirm_leave_room_dialog, ContextMenuBin},
+    components::{ContextMenuBin, confirm_leave_room_dialog},
     prelude::*,
     session::model::{
         IdentityVerification, ReceiptPosition, Room, RoomCategory, SidebarIconItem,
@@ -302,6 +302,23 @@ mod imp {
             let category = room.category();
 
             match category {
+                RoomCategory::Knocked => {
+                    action_group.add_action_entries([gio::ActionEntry::builder(
+                        "retract-invite-request",
+                    )
+                    .activate(clone!(
+                        #[weak(rename_to = imp)]
+                        self,
+                        move |_, _, _| {
+                            if let Some(room) = imp.room() {
+                                spawn!(async move {
+                                    imp.set_room_category(&room, TargetRoomCategory::Left).await;
+                                });
+                            }
+                        }
+                    ))
+                    .build()]);
+                }
                 RoomCategory::Invited => {
                     action_group.add_action_entries([
                         gio::ActionEntry::builder("accept-invite")
@@ -414,7 +431,21 @@ mod imp {
                         ))
                         .build()]);
 
-                    if !room.is_read() {
+                    if room.is_read() {
+                        action_group.add_action_entries([gio::ActionEntry::builder(
+                            "mark-as-unread",
+                        )
+                        .activate(clone!(
+                            #[weak]
+                            room,
+                            move |_, _, _| {
+                                spawn!(async move {
+                                    room.mark_as_unread().await;
+                                });
+                            }
+                        ))
+                        .build()]);
+                    } else {
                         action_group.add_action_entries([gio::ActionEntry::builder(
                             "mark-as-read",
                         )
