@@ -15,21 +15,23 @@ use crate::{
     prelude::*,
     toast,
     utils::{
+        File,
         media::{
+            FrameDimensions, MediaFileError,
             image::{
                 Blurhash, Image, ImageError, ImageRequestPriority, ImageSource,
                 ThumbnailDownloader, ThumbnailSettings,
             },
-            FrameDimensions, MediaFileError,
         },
-        save_data_to_tmp_file, File,
+        save_data_to_tmp_file,
     },
 };
 
 /// Get the filename of a media message.
 macro_rules! filename {
     ($message:ident, $mime_fallback:expr) => {{
-        let filename = $message.filename();
+        let mut filename = $message.filename().to_owned();
+        filename.clean_string();
 
         if filename.is_empty() {
             let mimetype = $message
@@ -39,7 +41,7 @@ macro_rules! filename {
 
             $crate::utils::media::filename_for_mime(mimetype, $mime_fallback)
         } else {
-            filename.to_owned()
+            filename
         }
     }};
 }
@@ -56,7 +58,7 @@ pub(crate) enum MediaMessage {
     /// A video.
     Video(VideoMessageEventContent),
     /// A sticker.
-    Sticker(StickerEventContent),
+    Sticker(Box<StickerEventContent>),
 }
 
 impl MediaMessage {
@@ -87,14 +89,31 @@ impl MediaMessage {
     /// The caption of the media, if any.
     ///
     /// Returns `Some((body, formatted_body))` if the media includes a caption.
-    pub(crate) fn caption(&self) -> Option<(&str, Option<&FormattedBody>)> {
-        match self {
-            Self::Audio(c) => c.caption().map(|caption| (caption, c.formatted.as_ref())),
-            Self::File(c) => c.caption().map(|caption| (caption, c.formatted.as_ref())),
-            Self::Image(c) => c.caption().map(|caption| (caption, c.formatted.as_ref())),
-            Self::Video(c) => c.caption().map(|caption| (caption, c.formatted.as_ref())),
+    pub(crate) fn caption(&self) -> Option<(String, Option<FormattedBody>)> {
+        let mut caption = match self {
+            Self::Audio(c) => c
+                .caption()
+                .map(|caption| (caption.to_owned(), c.formatted.clone())),
+            Self::File(c) => c
+                .caption()
+                .map(|caption| (caption.to_owned(), c.formatted.clone())),
+            Self::Image(c) => c
+                .caption()
+                .map(|caption| (caption.to_owned(), c.formatted.clone())),
+            Self::Video(c) => c
+                .caption()
+                .map(|caption| (caption.to_owned(), c.formatted.clone())),
             Self::Sticker(_) => None,
-        }
+        };
+
+        caption.take_if(|(caption, formatted)| {
+            caption.clean_string();
+            formatted.clean_string();
+
+            caption.is_empty()
+        });
+
+        caption
     }
 
     /// Fetch the content of the media with the given client.
@@ -104,7 +123,7 @@ impl MediaMessage {
         let media = client.media();
 
         macro_rules! content {
-            ($event_content:ident) => {{
+            ($event_content:expr) => {{
                 Ok(
                     $crate::spawn_tokio!(
                         async move { media.get_file(&$event_content, true).await }
@@ -121,7 +140,7 @@ impl MediaMessage {
             Self::File(c) => content!(c),
             Self::Image(c) => content!(c),
             Self::Video(c) => content!(c),
-            Self::Sticker(c) => content!(c),
+            Self::Sticker(c) => content!(*c),
         }
     }
 
@@ -181,7 +200,7 @@ impl MediaMessage {
                     toast!(parent, gettext("Could not access file"));
                 }
             }
-        };
+        }
     }
 }
 
@@ -199,7 +218,7 @@ impl From<FileMessageEventContent> for MediaMessage {
 
 impl From<StickerEventContent> for MediaMessage {
     fn from(value: StickerEventContent) -> Self {
-        Self::Sticker(value)
+        Self::Sticker(value.into())
     }
 }
 
@@ -211,7 +230,7 @@ pub(crate) enum VisualMediaMessage {
     /// A video.
     Video(VideoMessageEventContent),
     /// A sticker.
-    Sticker(StickerEventContent),
+    Sticker(Box<StickerEventContent>),
 }
 
 impl VisualMediaMessage {
@@ -373,6 +392,12 @@ impl From<VideoMessageEventContent> for VisualMediaMessage {
 
 impl From<StickerEventContent> for VisualMediaMessage {
     fn from(value: StickerEventContent) -> Self {
+        Self::Sticker(value.into())
+    }
+}
+
+impl From<Box<StickerEventContent>> for VisualMediaMessage {
+    fn from(value: Box<StickerEventContent>) -> Self {
         Self::Sticker(value)
     }
 }
