@@ -1,84 +1,69 @@
 //! Collection of common methods and types.
 
-mod dummy_object;
-pub mod expression;
-mod expression_list_model;
-pub(crate) mod key_bindings;
-mod location;
-pub mod macros;
-pub mod matrix;
-pub mod media;
-pub mod notifications;
-pub(crate) mod oauth;
-mod single_item_list_model;
-pub mod sourceview;
-pub mod string;
-pub mod template_callbacks;
-
 use std::{
+    borrow::Cow,
     cell::{Cell, OnceCell, RefCell},
-    fmt, fs,
-    io::{self, Write},
+    fmt, fs, io,
+    io::Write,
     path::{Path, PathBuf},
     rc::{Rc, Weak},
     sync::{Arc, LazyLock},
 };
 
-use futures_util::{
-    future::{self, Either, Future},
-    pin_mut,
-};
-use gtk::{gio, glib, prelude::*};
+use adw::prelude::*;
+use gtk::{gio, glib};
 use regex::Regex;
 use tempfile::NamedTempFile;
+use tokio::task::{AbortHandle, JoinHandle};
 
-pub use self::{
-    dummy_object::DummyObject,
+pub(crate) mod expression;
+mod expression_list_model;
+mod fixed_selection;
+mod grouping_list_model;
+pub(crate) mod key_bindings;
+mod location;
+mod macros;
+pub(crate) mod matrix;
+pub(crate) mod media;
+pub(crate) mod notifications;
+mod placeholder_object;
+mod single_item_list_model;
+pub(crate) mod sourceview;
+pub(crate) mod string;
+mod template_callbacks;
+pub(crate) mod toast;
+
+pub(crate) use self::{
     expression_list_model::ExpressionListModel,
+    fixed_selection::FixedSelection,
+    grouping_list_model::*,
     location::{Location, LocationError, LocationExt},
+    placeholder_object::PlaceholderObject,
     single_item_list_model::SingleItemListModel,
+    template_callbacks::TemplateCallbacks,
 };
 use crate::{PROFILE, RUNTIME};
 
-/// The path of the directory where data should be stored, depending on its
-/// type.
-pub fn data_dir_path(data_type: DataType) -> PathBuf {
-    let mut path = match data_type {
-        DataType::Persistent => glib::user_data_dir(),
-        DataType::Cache => glib::user_cache_dir(),
-    };
-    path.push(PROFILE.dir_name().as_ref());
-
-    path
-}
-
 /// The type of data.
 #[derive(Debug, Clone, Copy)]
-pub enum DataType {
+pub(crate) enum DataType {
     /// Data that should not be deleted.
     Persistent,
     /// Cache that can be deleted freely.
     Cache,
 }
 
-pub enum TimeoutFuture {
-    Timeout,
-}
+impl DataType {
+    /// The path of the directory where data should be stored, depending on this
+    /// type.
+    pub(crate) fn dir_path(self) -> PathBuf {
+        let mut path = match self {
+            DataType::Persistent => glib::user_data_dir(),
+            DataType::Cache => glib::user_cache_dir(),
+        };
+        path.push(PROFILE.dir_name().as_ref());
 
-/// Executes the given future with the given timeout.
-///
-/// If the future didn't resolve before the timeout was reached, this returns
-/// an `Err(TimeoutFuture)`.
-pub async fn timeout_future<T>(
-    timeout: std::time::Duration,
-    fut: impl Future<Output = T>,
-) -> Result<T, TimeoutFuture> {
-    let timeout = glib::timeout_future(timeout);
-    pin_mut!(fut);
-
-    match future::select(fut, timeout).await {
-        Either::Left((x, _)) => Ok(x),
-        Either::Right(_) => Err(TimeoutFuture::Timeout),
+        path
     }
 }
 
@@ -86,18 +71,18 @@ pub async fn timeout_future<T>(
 ///
 /// The expected format to replace is `{name}`, where `name` is the first string
 /// in the dictionary entry tuple.
-pub fn freplace(s: String, args: &[(&str, &str)]) -> String {
-    let mut s = s;
+pub(crate) fn freplace<'a>(s: &'a str, args: &[(&str, &str)]) -> Cow<'a, str> {
+    let mut s = Cow::Borrowed(s);
 
     for (k, v) in args {
-        s = s.replace(&format!("{{{k}}}"), v);
+        s = Cow::Owned(s.replace(&format!("{{{k}}}"), v));
     }
 
     s
 }
 
 /// Regex that matches a string that only includes emojis.
-pub static EMOJI_REGEX: LazyLock<Regex> = LazyLock::new(|| {
+pub(crate) static EMOJI_REGEX: LazyLock<Regex> = LazyLock::new(|| {
     Regex::new(
         r"(?x)
         ^
@@ -114,7 +99,7 @@ pub static EMOJI_REGEX: LazyLock<Regex> = LazyLock::new(|| {
 
 /// Inner to manage a bound object.
 #[derive(Debug)]
-pub struct BoundObjectInner<T: ObjectType> {
+struct BoundObjectInner<T: ObjectType> {
     obj: T,
     signal_handler_ids: Vec<glib::SignalHandlerId>,
 }
@@ -137,7 +122,7 @@ impl<T: ObjectType> BoundObject<T> {
     ///
     /// Calls `disconnect_signals` first to drop the previous strong reference
     /// and disconnect the previous signal handlers.
-    pub fn set(&self, obj: T, signal_handler_ids: Vec<glib::SignalHandlerId>) {
+    pub(crate) fn set(&self, obj: T, signal_handler_ids: Vec<glib::SignalHandlerId>) {
         self.disconnect_signals();
 
         let inner = BoundObjectInner {
@@ -208,7 +193,7 @@ impl<T: ObjectType> BoundObjectWeakRef<T> {
     ///
     /// Calls `disconnect_signals` first to remove the previous weak reference
     /// and disconnect the previous signal handlers.
-    pub fn set(&self, obj: &T, signal_handler_ids: Vec<glib::SignalHandlerId>) {
+    pub(crate) fn set(&self, obj: &T, signal_handler_ids: Vec<glib::SignalHandlerId>) {
         self.disconnect_signals();
 
         self.weak_obj.set(Some(obj));
@@ -279,7 +264,7 @@ impl<T: ObjectType> BoundConstructOnlyObject<T> {
     /// Set the given object and signal handlers IDs.
     ///
     /// Panics if the object was already set.
-    pub fn set(&self, obj: T, signal_handler_ids: Vec<glib::SignalHandlerId>) {
+    pub(crate) fn set(&self, obj: T, signal_handler_ids: Vec<glib::SignalHandlerId>) {
         self.obj.set(obj).unwrap();
         self.signal_handler_ids.replace(signal_handler_ids);
     }
@@ -344,7 +329,7 @@ impl<T> OngoingAsyncAction<T> {
     /// Create a new async action that sets the given value.
     ///
     /// Returns both a strong and a weak reference.
-    pub fn set(value: T) -> (Self, WeakOngoingAsyncAction<T>) {
+    pub(crate) fn set(value: T) -> (Self, WeakOngoingAsyncAction<T>) {
         let strong = Rc::new(AsyncAction::Set(value));
         let weak = Rc::downgrade(&strong);
         (Self { strong }, WeakOngoingAsyncAction { weak })
@@ -353,25 +338,14 @@ impl<T> OngoingAsyncAction<T> {
     /// Create a new async action that removes a value.
     ///
     /// Returns both a strong and a weak reference.
-    pub fn remove() -> (Self, WeakOngoingAsyncAction<T>) {
+    pub(crate) fn remove() -> (Self, WeakOngoingAsyncAction<T>) {
         let strong = Rc::new(AsyncAction::Remove);
         let weak = Rc::downgrade(&strong);
         (Self { strong }, WeakOngoingAsyncAction { weak })
     }
 
-    /// Create a new weak reference to this async action.
-    pub fn downgrade(&self) -> WeakOngoingAsyncAction<T> {
-        let weak = Rc::downgrade(&self.strong);
-        WeakOngoingAsyncAction { weak }
-    }
-
-    /// The inner action.
-    pub fn action(&self) -> &AsyncAction<T> {
-        &self.strong
-    }
-
     /// Get the inner value, if any.
-    pub fn as_value(&self) -> Option<&T> {
+    pub(crate) fn as_value(&self) -> Option<&T> {
         self.strong.as_value()
     }
 }
@@ -392,7 +366,7 @@ impl<T> WeakOngoingAsyncAction<T> {
 
 /// An async action.
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub enum AsyncAction<T> {
+pub(crate) enum AsyncAction<T> {
     /// An async action is ongoing to set this value.
     Set(T),
 
@@ -432,7 +406,7 @@ impl<T> TokioDrop<T> {
     /// Sets the contents of this cell to `value`.
     ///
     /// Returns `Ok(())` if the cell was empty and `Err(value)` if it was full.
-    pub fn set(&self, value: T) -> Result<(), T> {
+    pub(crate) fn set(&self, value: T) -> Result<(), T> {
         self.0.set(value)
     }
 }
@@ -492,7 +466,7 @@ pub enum LoadingState {
 }
 
 /// Convert the given checked `bool` to a `GtkAccessibleTristate`.
-pub fn bool_to_accessible_tristate(checked: bool) -> gtk::AccessibleTristate {
+pub(crate) fn bool_to_accessible_tristate(checked: bool) -> gtk::AccessibleTristate {
     if checked {
         gtk::AccessibleTristate::True
     } else {
@@ -589,7 +563,7 @@ struct InnerCountedRef {
 
 impl CountedRef {
     /// Construct a counted reference.
-    pub fn new<F1, F2>(on_zero: F1, on_non_zero: F2) -> Self
+    pub(crate) fn new<F1, F2>(on_zero: F1, on_non_zero: F2) -> Self
     where
         F1: Fn() + 'static,
         F2: Fn() + 'static,
@@ -641,3 +615,115 @@ impl Drop for CountedRef {
         }
     }
 }
+
+/// Extensions trait for types with a `child` property.
+pub(crate) trait ChildPropertyExt {
+    /// The child of this widget, is any.
+    fn child_property(&self) -> Option<gtk::Widget>;
+
+    /// Set the child of this widget.
+    fn set_child_property(&self, child: Option<&impl IsA<gtk::Widget>>);
+
+    /// Get the child if it is of the proper type, or construct it with the
+    /// given function and set is as the child of this widget before returning
+    /// it.
+    fn child_or_else<W>(&self, f: impl FnOnce() -> W) -> W
+    where
+        W: IsA<gtk::Widget>,
+    {
+        if let Some(child) = self.child_property().and_downcast() {
+            child
+        } else {
+            let child = f();
+            self.set_child_property(Some(&child));
+            child
+        }
+    }
+
+    /// Get the child if it is of the proper type, or construct it with its
+    /// `Default` implementation and set is as the child of this widget before
+    /// returning it.
+    fn child_or_default<W>(&self) -> W
+    where
+        W: IsA<gtk::Widget> + Default,
+    {
+        self.child_or_else(Default::default)
+    }
+}
+
+impl<W> ChildPropertyExt for W
+where
+    W: IsABin,
+{
+    fn child_property(&self) -> Option<gtk::Widget> {
+        self.child()
+    }
+
+    fn set_child_property(&self, child: Option<&impl IsA<gtk::Widget>>) {
+        self.set_child(child);
+    }
+}
+
+impl ChildPropertyExt for gtk::ListItem {
+    fn child_property(&self) -> Option<gtk::Widget> {
+        self.child()
+    }
+
+    fn set_child_property(&self, child: Option<&impl IsA<gtk::Widget>>) {
+        self.set_child(child);
+    }
+}
+
+/// Helper trait to implement for widgets that subclass `AdwBin`, to be able to
+/// use the `ChildPropertyExt` trait.
+///
+/// This trait is to circumvent conflicts in Rust's type system, where if we try
+/// to implement `ChildPropertyExt for W where W: IsA<adw::Bin>` it complains
+/// that the other external types that implement `ChildPropertyExt` might
+/// implement `IsA<adw::Bin>` in the future… So instead of reimplementing
+/// `ChildPropertyExt` for every type where we need it, which requires to
+/// implement two methods, we only implement this which requires nothing.
+pub(crate) trait IsABin: IsA<adw::Bin> {}
+
+impl IsABin for adw::Bin {}
+
+/// A wrapper around [`JoinHandle`] that aborts the future if it is dropped
+/// before the task ends.
+///
+/// The main API for this type is [`AbortableHandle::await_task()`].
+#[derive(Debug, Default)]
+pub(crate) struct AbortableHandle {
+    abort_handle: RefCell<Option<AbortHandle>>,
+}
+
+impl AbortableHandle {
+    /// Await the task of the given `JoinHandle`.
+    ///
+    /// Aborts the previous task that was running, if any.
+    ///
+    /// Returns `None` if the task was aborted before completion.
+    pub(crate) async fn await_task<T>(&self, join_handle: JoinHandle<T>) -> Option<T> {
+        self.abort();
+
+        self.abort_handle.replace(Some(join_handle.abort_handle()));
+
+        let result = join_handle.await.ok();
+
+        self.abort_handle.take();
+
+        result
+    }
+
+    /// Abort the current task, if possible.
+    pub(crate) fn abort(&self) {
+        if let Some(abort_handle) = self.abort_handle.take() {
+            abort_handle.abort();
+        }
+    }
+}
+
+impl Drop for AbortableHandle {
+    fn drop(&mut self) {
+        self.abort();
+    }
+}
