@@ -10,9 +10,9 @@ use url::Url;
 #[cfg(test)]
 mod tests;
 
-use super::matrix::{find_at_room, MatrixIdUri, AT_ROOM};
+use super::matrix::{AT_ROOM, MatrixIdUri, find_at_room};
 use crate::{
-    components::{LabelWithWidgets, Pill},
+    components::{AvatarImageSafetySetting, LabelWithWidgets, Pill},
     prelude::*,
     session::model::Room,
 };
@@ -87,6 +87,18 @@ pub(crate) trait StrMutExt {
 
     /// Append an ellipsis, except if this string already ends with an ellipsis.
     fn append_ellipsis(&mut self);
+
+    /// Strip the NUL byte.
+    ///
+    /// Since they are used by GTK as the end of a string, strings in properties
+    /// will be truncated at the first NUL byte.
+    fn strip_nul(&mut self);
+
+    /// Remove unnecessary or problematic characters from the string.
+    fn clean_string(&mut self) {
+        self.strip_nul();
+        self.truncate_end_whitespaces();
+    }
 }
 
 impl StrMutExt for String {
@@ -125,6 +137,35 @@ impl StrMutExt for String {
             self.push('…');
         }
     }
+
+    fn strip_nul(&mut self) {
+        self.retain(|c| c != '\0');
+    }
+}
+
+/// Extensions to `Option<String>`.
+pub(crate) trait OptionStringExt: Sized {
+    /// Remove unnecessary or problematic characters from the string.
+    ///
+    /// If the final string is empty, replaces it with `None`.
+    fn clean_string(&mut self);
+
+    /// Remove unnecessary or problematic characters from the string.
+    ///
+    /// If the final string is empty, replaces it with `None`.
+    fn into_clean_string(mut self) -> Self {
+        self.clean_string();
+        self
+    }
+}
+
+impl OptionStringExt for Option<String> {
+    fn clean_string(&mut self) {
+        self.take_if(|s| {
+            s.clean_string();
+            s.is_empty()
+        });
+    }
 }
 
 /// Common extensions for adding Pango markup to mutable strings.
@@ -141,7 +182,7 @@ pub(crate) trait PangoStrMutExt {
     ///
     /// Returns the created [`Pill`], it the URI was added as a mention.
     fn maybe_append_mention(&mut self, uri: impl TryInto<MatrixIdUri>, room: &Room)
-        -> Option<Pill>;
+    -> Option<Pill>;
 
     /// Append the given string and replace `@room` with a mention.
     ///
@@ -182,7 +223,9 @@ impl PangoStrMutExt for String {
             self.push_str(LabelWithWidgets::PLACEHOLDER);
             self.push_str(&(&s[pos + AT_ROOM.len()..]).escape_markup());
 
-            Some(room.at_room().to_pill())
+            // We do not need to watch safety settings for mentions, rooms will be watched
+            // automatically.
+            Some(room.at_room().to_pill(AvatarImageSafetySetting::None, None))
         } else {
             self.push_str(&s.escape_markup());
             None
